import os
import json

from src.Model.Data_Storage.DataSingleton_Model import DataSingleton
from src.Model.Data_Storage.Filters_Model import Filters_Infos

class Project_Infos(metaclass=DataSingleton):
    """
    A class to manage and store information about the project

        - Absolute Path to Files (Data, FLUT, Filters)
        - Previous files opened
        - Export Project (JSON)
    """
    def __init__(self):
        
        self.filters = Filters_Infos()
        self.previousFiles = {"DataFiles": {}, 
                              "NameFiles": {}, 
                              "FlutFiles": [], 
                              "FiltersFiles": [], 
                              "ProjectFiles": []}
        
        self.InitReset()

    def InitReset(self):
        """
        Reset the current file paths to None. This is useful for initializing the state
        of a new or empty project.
        """

        self.currentDataFile = None
        self.currentNameFile = None
        self.currentFlutFile = None
        self.currentFiltersFile = None

    def InitRestore(self, projectSave: 'Project_Infos'):
        """
        Restore the state of the current project using a saved Project_Infos object.

        Parameters
        ----------
        projectSave : Project_Infos
            An instance of Project_Infos containing the saved state to be restored.
        """

        self.currentDataFile = projectSave.currentDataFile
        self.currentNameFile = projectSave.currentNameFile
        self.currentFlutFile = projectSave.currentFlutFile

    def ExportToJSON(self, filePathFull):

        """
        Export the current project information to a JSON file. This includes saving the 
        current filter settings to a separate file.

        Parameters
        ----------
        filePathFull : str
            The full path where the project JSON file will be saved.
        """

        filePath = os.path.dirname(filePathFull)
        filePath_basename = ".".join(os.path.basename(filePathFull).split('.')[:-1])
        newFilePath = os.path.join(filePath, f"{filePath_basename}-Filters.json")
        self.filters.ExportToJSON(newFilePath)

        projectToJSON = {

            "currentDataFile": self.currentDataFile,
            "currentNameFile": self.currentNameFile,
            "currentFlutFile": self.currentFlutFile,
            "currentFiltersFile": newFilePath
        }

        with open(filePathFull, 'w', encoding='utf-8') as file:
            json.dump(projectToJSON, file, ensure_ascii=False, indent=4)


import numpy as np

from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.QtWidgets import QTableWidgetItem

class RecovertListData_Worker(QObject):
    finished = pyqtSignal()
    progress = pyqtSignal(list)

    def __init__(self, connGraph, filters):
        super().__init__()

        self.connGraph = connGraph
        self.filters = filters

    def run(self):

        # Build a list of edges with each information needed
        edges = np.array(self.connGraph.GetEdgesDetails())

        # Prepare formatting for int/float values in the tableWidget
        # Note : It is important to not convert int values into str() values without .format
        #        otherwise sorting will not work properly for these values
        intMax = max(np.concatenate((edges[:, 0], edges[:, 1])))
        intFormat = "{:" + str(intMax) + "d}"
        floatFormat = "{:10." + str(self.filters.valueRound) + "f}"

        # Create each row with edges info
        for edge in edges:

            node1 = intFormat.format(int(edge[0]))
            node2 = intFormat.format(int(edge[1]))
            area1 = str(edge[2])
            area2 = str(edge[3])
            region1 = str(edge[4])
            region2 = str(edge[5])
            connType = str(edge[6])
            value = floatFormat.format(float(edge[7]))
            rank = intFormat.format(int(edge[8]))
            newRow = [node1, node2, area1, area2, region1, region2, connType, value, rank]
            self.progress.emit(newRow)

        self.finished.emit()


import math
import math
import random
import sys
from copy import deepcopy
from math import comb
from typing import Union

import numpy as np
import pyqtgraph as pg
from PyQt5.QtCore import QPointF, Qt, QSize
from PyQt5.QtGui import QFont, QColor, QPainter, QTransform, QImage, QPainterPath, QFontMetrics
from PyQt5.QtWidgets import QGraphicsBlurEffect, QWidget, QVBoxLayout
from matplotlib.path import Path

from src.Model.Data_Storage.ConnGraph_Model import ConnGraph_Infos
from src.Model.Data_Storage.Filters_Model import Filters_Infos


class RegionLabelItem_GTGraphic(pg.ImageItem):

    def __init__(self, text: str, coordinates, color: QColor = QColor(0, 0, 0), **kargs):
        # Init label attributes
        self.text = text
        self.coordinates = coordinates
        self.color = color
        self.shaded = False
        self.distance = 1  # TODO : à voir quel réglage est préférable

        size_min, size_max = 8, 12
        self.size = self.ComputeTextSize(size_min, size_max)
        self.rotation, self.anchor = self.ComputeTextRotation()

        np_image = self._InitText()
        super().__init__(np_image, **kargs)

        self._InitImage()

    def _InitText(self):
        # Initialisation de la police pour calculer la taille de la zone de texte
        font = QFont("Arial", self.size, weight=QFont.Bold)
        self.text_width = QFontMetrics(font).width(self.text, len(self.text))
        self.text_height = QFontMetrics(font).height() + 6

        # width, height = self.size * len(self.text), self.size + 6
        image = QImage(self.text_width, self.text_height, QImage.Format_ARGB32)
        image.fill(Qt.transparent)

        # On utilise QPainter et QPainterPath pour afficher le texte
        painter = QPainter(image)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setRenderHint(QPainter.TextAntialiasing)
        painter.setFont(font)

        path = QPainterPath()
        path.addText(0, self.size + 3, font, self.text)

        painter.fillPath(path, self.color)
        painter.end()

        # On convertit ensuite ce texte en matrice pour initialiser l'ImageItem
        np_image = self.ImageToNpArray(image)

        return np_image

    def _InitImage(self):
        self.setPos(self.ComputeTextCoordinates(self.distance))

        self.setTransform(self.ComputeTextTransform())

        blurEffect = QGraphicsBlurEffect(blurRadius=1.01)
        self.setGraphicsEffect(blurEffect)

    def ImageToNpArray(self, image: QImage):
        width, height = image.width(), image.height()

        buffer = image.bits().asstring(width * height * 4)
        np_array = np.frombuffer(buffer, dtype=np.uint8).reshape((height, width, 4))

        return np_array

    def ComputeTextSize(self, size_min: int, size_max: int):

        # Compute text size based on the width of text area
        size = int(math.dist(self.coordinates[2], self.coordinates[3]))
        # To guarantee that text size is beetween given min and max size
        size = min(size_max, size)
        size = max(size_min, size)

        return size

    def ComputeTextRotation(self):
        if self.coordinates[2, 1] - self.coordinates[3, 1] < 0:
            x = -np.mean(self.coordinates[:, 0])
            y = -np.mean(self.coordinates[:, 1])
            anchor = (1, 0.5)
        else:
            x = np.mean(self.coordinates[:, 0])
            y = np.mean(self.coordinates[:, 1])
            anchor = (0, 0.5)

        angle = -np.arctan2(x, y)

        return np.degrees(angle), anchor

    def ComputeTextCoordinates(self, distance):
        p1, p2 = self.coordinates[2], self.coordinates[3]

        # Calculate the normal vector to the longer side
        dx, dy = p2 - p1
        normal_vector = np.array([-dy, dx])
        normal_vector = normal_vector / np.linalg.norm(normal_vector) * distance

        # Determine the direction to place the point outside the region
        midpoint = (p1 + p2) / 2
        point = midpoint + normal_vector

        text_coordinates = QPointF(point[0], point[1])

        # Now the position is defined, we translate the text depending on the anchor settings

        x = text_coordinates.x()
        text_coordinates.setX(x)
        return text_coordinates

    def ComputeTextTransform(self):
        tr = QTransform()
        tr.scale(0.3, 0.3)
        tr.rotate(self.rotation)

        xt = -self.anchor[1] * self.text_height
        # We use text_len to put the anchor at the end of the text if it is reversed
        yt = -self.anchor[0] * self.text_width + self.distance
        tr.translate(xt, yt)

        return tr

    """
    Si les connexions ne sont pas affichées : gris
    Si les connexions sont partiellement affichées : gris
    Si les connexions sont toutes affichées : noir
    """

    def SetTextShaded(self, shaded: bool):
        self.shaded = shaded
        if shaded:
            self.setOpacity(0.7)
        else:
            self.setOpacity(1)

    def IsPointInTextPolygon(self, x, y):
        polygon = np.array([
            self.coordinates[0],
            self.coordinates[1],
            self.coordinates[2],
            self.coordinates[3]
        ])
        path = Path(polygon)

        return path.contains_point((x, y))


class RegionItem_GTGraphic:
    outside: pg.PlotDataItem  # Outer line of the region
    inside: pg.FillBetweenItem  # Colored part of the region

    interRegionOutside: pg.PlotDataItem  # Outer line of the region
    interRegionInside: pg.FillBetweenItem  # InterRegional colored part (between the region and text area)

    gtValueOutside : pg.PlotDataItem  # Outer line of the value polygon
    gtValueInside : pg.FillBetweenItem  # Value polygon colored part

    text: RegionLabelItem_GTGraphic  # Text label

    def __init__(self, code: int, name: str, value : float, maxValue : float,
                 coordinates,  # coordinates of each item : region, interRegion, label, gtValue
                 circlePoints,  # 4 list of corresponding circle coords for each item
                 precision=20,
                 regionColor=QColor(255, 255, 255, 255),  # inside color of a region, white by default
                 outlineColor=QColor(0, 0, 0, 255),  # outline color of a region, black by default
                 interRegionColor=QColor(255, 255, 255, 255),  # color of an inter-region associated to a region
                 *args):
        """
        :param coordinates: 4 coordinates points which define the polygon to draw
        """
        self.code = code  # Code of the region
        self.name = name  # Name of the region
        self.value = value
        self.maxValue = maxValue

        self.coordinates = coordinates  # Coordinates of region, interRegion and label
        self.regionColor = regionColor  # Region color
        self.outlineColor = outlineColor  # Region color (outline)
        self.interRegionColor = interRegionColor  # Inter region color
        self.precision = precision  # Precision value. WARING : Greatly affect performances
        self.allCircles = circlePoints  # Circles points for drawing region, interRegion and label

        self._ComputePolygons()
        self._DrawRegion()
        self._DrawInterRegion()
        self._DrawLabel()
        self._DrawGtValue()

        super().__init__(*args)

    def _ComputePolygons(self):
        # Compute polygon lines
        # regions
        self.innerCoords = self.LineToCircleArc(self.allCircles[0][0], self.coordinates[0][0])
        self.outerCoords = self.LineToCircleArc(self.allCircles[0][1], self.coordinates[0][3])

        # inter regions
        self.innerInterRegCoords = self.LineToCircleArc(self.allCircles[1][0], self.coordinates[1][0])
        self.outerInterRegCoords = self.LineToCircleArc(self.allCircles[1][1], self.coordinates[1][3])

        # value
        coeff = 1 - (self.value / self.maxValue)
        self.outerGtValueCoords = self.LineToCircleArc(self.allCircles[3][1], self.coordinates[3][1])
        self.innerGtValueCoords = self.outerGtValueCoords * coeff

        # For gtValues polygons which begins at the center of the graph
        """circleCoords = self.allCircles[3][1] * (1 - coeff)
        self.innerGtValueCoords = np.array([[0, 0], [0, 0]])
        self.outerGtValueCoords = self.LineToCircleArc(circleCoords, self.coordinates[3][1])"""

    def _DrawRegion(self):
        x_coords = np.append(np.append(self.innerCoords[:, 0],
                                       self.outerCoords[::-1, 0]),
                             self.innerCoords[0, 0])

        y_coords = np.append(np.append(self.innerCoords[:, 1],
                                       self.outerCoords[::-1, 1]),
                             self.innerCoords[0, 1])

        # Draw the outline of the polygon
        # TODO : width
        self.outside = pg.PlotDataItem(x_coords, y_coords, pen=pg.mkPen(color=self.outlineColor, width=1))

        # Color the inside of the polygon
        innerLine = pg.PlotDataItem(self.innerCoords[:, 0], self.innerCoords[:, 1])
        outerLine = pg.PlotDataItem(self.outerCoords[:, 0], self.outerCoords[:, 1])

        self.inside = pg.FillBetweenItem(innerLine, outerLine, brush=pg.mkBrush(self.regionColor))

    def _DrawInterRegion(self):

        x_coords = np.append(np.append(self.innerInterRegCoords[:, 0],
                                       self.outerInterRegCoords[::-1, 0]),
                             self.innerInterRegCoords[0, 0])

        y_coords = np.append(np.append(self.innerInterRegCoords[:, 1],
                                       self.outerInterRegCoords[::-1, 1]),
                             self.innerInterRegCoords[0, 1])


        # Draw the outline of the polygon
        self.interRegionOutside = pg.PlotDataItem(x_coords, y_coords, pen=pg.mkPen(color=self.interRegionColor, width=1))


        # Color the inside of the polygon
        innerLine = pg.PlotDataItem(self.innerInterRegCoords[:, 0], self.innerInterRegCoords[:, 1])
        outerLine = pg.PlotDataItem(self.outerInterRegCoords[:, 0], self.outerInterRegCoords[:, 1])

        self.interRegionInside = pg.FillBetweenItem(innerLine, outerLine, brush=pg.mkBrush(self.interRegionColor))

    def _DrawGtValue(self):
        x_coords = np.append(np.append(self.innerGtValueCoords[:, 0],
                                       self.outerGtValueCoords[::-1, 0]),
                             self.innerGtValueCoords[0, 0])

        y_coords = np.append(np.append(self.innerGtValueCoords[:, 1],
                                       self.outerGtValueCoords[::-1, 1]),
                             self.innerGtValueCoords[0, 1])

        # Draw the outline of the polygon
        #self.gtOutlineColor = QColor(150, 150, 150)
        self.gtValueOutside = pg.PlotDataItem(x_coords, y_coords, pen=pg.mkPen(color=self.outlineColor, width=1))


        # Color the inside of the polygon
        innerLine = pg.PlotDataItem(self.innerGtValueCoords[:, 0], self.innerGtValueCoords[:, 1])
        outerLine = pg.PlotDataItem(self.outerGtValueCoords[:, 0], self.outerGtValueCoords[:, 1])

        """color = self.regionColor
        color.setAlpha(int(color.alpha() * 0.6))"""

        self.gtValueInside = pg.FillBetweenItem(innerLine, outerLine, brush=pg.mkBrush(self.regionColor))

    def _DrawLabel(self):
        # Draw text next to the polygon
        self.text = RegionLabelItem_GTGraphic(self.name, self.coordinates[2])

    # Function used for rouding region borders
    def LineToCircleArc(self, circlePoints, start):
        # Find the two points on the circle closest to the start and end points
        startDistances = np.linalg.norm(circlePoints - start, axis=1)

        startIndex = np.argmin(startDistances)
        endIndex = startIndex + self.precision

        return circlePoints[startIndex:endIndex + 1]

    #TODO : exemple de commentaire pour la doc ici !!
    def IsPointInRegionPolygon(self, x, y):
        """
        Compute if the given point is inside the polygon region

        Parameters
        ----------
        x : int
            x coordinate of the point
        y : int
            y coordinate of the point

        Returns
        -------
        bool
            True if inside else False
        """
        # Create matplotlib Path object
        polygonPoints = np.vstack((self.innerCoords, self.outerCoords[::-1]))
        polygonPath = Path(polygonPoints)

        # Check if the point is inside the polygon
        return polygonPath.contains_point((x, y))


class ConnGraphicView(pg.ViewBox):
    graph_info: ConnGraph_Infos
    filters: Filters_Infos
    regions: list[Union[RegionItem_GTGraphic, None]]  # Can contain both None or RegionItem_GTGraphic
    colors: list[QColor]
    graphicFilter: bool  # True if a region is highlighted or hidden by a click on the chart
    highlightedRegions: bool

    graph_info: ConnGraph_Infos

    # TODO : Modifier les paramètres d'initialisation pour passer le graphe complet
    def __init__(self, colorSet: list[QColor], parent=None):

        super().__init__(parent)

        self.graph_info = ConnGraph_Infos()
        filters = Filters_Infos()

        numPoints = len(self.graph_info.areasOrder) + 2  # For blank at begin and end

        # Init regions list
        self.regions = [None for i in range(numPoints)]
        self.highlightedRegions = False

        # Init class attributes
        self.filters = deepcopy(filters)  # Save the current state of the filter
        self.graphicFilter = False
        self.numPoints = numPoints
        self.maxAbsConnValue = self.graph_info.absMinMax[1]
        self.colorSet = colorSet

        self._InitViewSettings()
        self._InitGraphSettings()
        self._GenerateGraphValues()
        self._InitGraph_Regions()

    def _InitViewSettings(self):

        self.disableAutoRange()
        self.setMouseEnabled(False, False)  # Disable mouse interactions
        self.setBackgroundColor("w")

        XRange = 100
        YRange = 100
        self.setRange(xRange=(-XRange, XRange), yRange=(-YRange, YRange))

    def _InitGraphSettings(self):

        self.radius = 50  # Radius in pyqtgraph unit of the text on the plot

        self.regionThickness = 300 / self.numPoints  # The more regions on the graph, the less the thickness of the donut will be

        self.interRegionThickness = self.regionThickness / 2
        self.interRegionDistance = self.regionThickness + 1  # Distance between region polygon and interRegion polygon

        self.textDistance = self.interRegionDistance + self.interRegionThickness + 0  # Distance between interRegion polygon and regionLabel item
        self.textLength = 20  # Lenght in pyqtgraph unit of the text on the plot

        self.gtValueDistance = 1  # Distance beetween region polygon and gtValue polygon

        # IMPORTANT : GREATLY AFFECT PERFORMANCES, but improves region smoothing
        self.precision = 20

        # Changing the font can make the text too long and go outside the plotting area
        self.font = QFont("Arial", pointSize=25, weight=QFont.ExtraBold)

    def _GenerateGraphValues(self):
        angles = np.linspace(0.5 * np.pi, 2.5 * np.pi, self.numPoints, endpoint=False)

        # coordinates of each region
        self.innerPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                            * self.radius)
        self.innerPoints = np.concatenate((self.innerPoints, [self.innerPoints[0]]))

        self.outerPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                            * (self.radius + self.regionThickness))
        self.outerPoints = np.concatenate((self.outerPoints, [self.outerPoints[0]]))

        # coordinates of each interRegion polygon
        self.interRegionInnerPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                                       * (self.radius + self.interRegionDistance))
        self.interRegionInnerPoints = np.concatenate((self.interRegionInnerPoints, [self.interRegionInnerPoints[0]]))

        self.interRegionOuterPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                                       * (self.radius + self.interRegionDistance + self.interRegionThickness))
        self.interRegionOuterPoints = np.concatenate((self.interRegionOuterPoints, [self.interRegionOuterPoints[0]]))

        # coordinates of each label
        self.innerTextPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                                * (self.radius + self.textDistance))
        self.innerTextPoints = np.concatenate((self.innerTextPoints, [self.innerTextPoints[0]]))

        self.outerTextPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                                * (self.radius + self.textDistance + self.textLength))
        self.outerTextPoints = np.concatenate((self.outerTextPoints, [self.outerTextPoints[0]]))

        # coordinates of each value triangle
        self.gtValueOuterPoints = (np.vstack((np.cos(angles), np.sin(angles))).T
                                       * (self.radius - self.gtValueDistance))
        self.gtValueOuterPoints = np.concatenate((self.gtValueOuterPoints, [self.gtValueOuterPoints[0]]))


        # create two circle for the inner and outer line of the donut
        circleAngle = np.linspace(0.5 * np.pi, 2.5 * np.pi, self.numPoints * self.precision, endpoint=False)
        circleAngle = np.append(circleAngle, circleAngle[0])

        circlesPoints = np.vstack((np.cos(circleAngle), np.sin(circleAngle))).T

        # Region polygon circles
        innerRegionCirclePoints = circlesPoints * self.radius
        outerRegionCirclePoints = circlesPoints * (self.radius + self.regionThickness)

        # InterRegion polygon circles
        innerInterRegionCirclePoints = circlesPoints * (self.radius + self.interRegionDistance)
        outerInterRegionCirclePoints = circlesPoints * (self.radius + self.interRegionDistance + self.interRegionThickness)

        # Text circles
        innerCircleTextPoints = circlesPoints * (self.radius + self.textDistance)
        outerCircleTextPoints = circlesPoints * (self.radius + self.textDistance + self.textLength)

        innerGtValuePoint = (0, 0)
        outerGtValueCirclePoints = circlesPoints * (self.radius - self.gtValueDistance)

        self.allCircles = ((innerRegionCirclePoints, outerRegionCirclePoints),
                           (innerInterRegionCirclePoints, outerInterRegionCirclePoints),
                           (innerCircleTextPoints, outerCircleTextPoints),
                           (innerGtValuePoint, outerGtValueCirclePoints))

    def _InitGraph_Regions(self):

        pos = 1
        for region in self.graph_info.areasOrder:
            pos += 1

            # TODO : maybe use region code == 0 instead
            region_name = region[0]
            if region_name == "xxxx":
                continue

            region_info = self.graph_info.areaInfos[region_name]

            # TODO : get inter region color here
            interRegionColor = self.graph_info.colorMajorRegions[region_info["MajorRegion"]]
            interRegionColor = [int(color * 255) for color in interRegionColor]

            self.CreateRegionOnGraph(self.graph_info.GetRegionIDWithName(region_name),
                                     region_name, region_info["RGBA"], interRegionColor, pos)

    # Get a region of the graph with its code
    def GetRegion(self, code):
        for region in self.regions:
            if region is not None:
                if region.code == code:
                    return region
        return None


    # ---------- Accessors ----------

    # create a new RegionItem at a given position in RegionList (beetween 1 and numPoints)
    def CreateRegionOnGraph(self, code, name, regionColor, interRegionColor, pos):
        pos -= 1

        # Get coordinates on the inner/outer circle of the graph
        regionCoordinates = np.array([
            [self.outerPoints[pos][0], self.outerPoints[pos][1]],
            [self.outerPoints[pos + 1][0], self.outerPoints[pos + 1][1]],
            [self.innerPoints[pos + 1][0], self.innerPoints[pos + 1][1]],
            [self.innerPoints[pos][0], self.innerPoints[pos][1]]
        ])

        interRegionCoordinates = np.array([
            [self.interRegionOuterPoints[pos][0], self.interRegionOuterPoints[pos][1]],
            [self.interRegionOuterPoints[pos + 1][0], self.interRegionOuterPoints[pos + 1][1]],
            [self.interRegionInnerPoints[pos + 1][0], self.interRegionInnerPoints[pos + 1][1]],
            [self.interRegionInnerPoints[pos][0], self.interRegionInnerPoints[pos][1]]
        ])

        textCoordinates = np.array([
            [self.outerTextPoints[pos][0], self.outerTextPoints[pos][1]],
            [self.outerTextPoints[pos + 1][0], self.outerTextPoints[pos + 1][1]],
            [self.innerTextPoints[pos + 1][0], self.innerTextPoints[pos + 1][1]],
            [self.innerTextPoints[pos][0], self.innerTextPoints[pos][1]]
        ])

        gtValueCoordinates = np.array([
            [self.gtValueOuterPoints[pos + 1][0], self.gtValueOuterPoints[pos + 1][1]],
            [self.gtValueOuterPoints[pos][0], self.gtValueOuterPoints[pos][1]]
        ])

        allCoordinates = (regionCoordinates, interRegionCoordinates, textCoordinates, gtValueCoordinates)

        # Create region
        # TODO : add transparency if needed

        # Convert RGB color values to QColor object
        regionColor = QColor(regionColor[0], regionColor[1], regionColor[2])
        interRegionColor = QColor(interRegionColor[0], interRegionColor[1], interRegionColor[2])

        #outlineColor = QColor(color[2], color[1], color[0])

        # TODO : delete this
        value = random.randint(0, 68000)
        maxValue = 68000

        self.regions[pos] = RegionItem_GTGraphic(code, name,
                                                 value, maxValue,
                                                 allCoordinates,
                                                 self.allCircles,
                                                 precision=self.precision,
                                                 regionColor=regionColor,
                                                 interRegionColor=interRegionColor)
        self.AddRegion(self.regions[pos], pos)

    def AddRegion(self, region: RegionItem_GTGraphic, pos=None):

        # TODO : à revoir une fois le graphe implémenté
        if pos is None:
            self.regions.append(region)
        else:
            self.regions[pos] = region

        # Add region polygon the gtGraphicView
        self.addItem(region.inside)
        self.addItem(region.outside)

        # Add interRegion polygon to the gtGraphicView
        self.addItem(region.interRegionInside)
        self.addItem(region.interRegionOutside)

        # Add label to the gtGraphicView
        self.addItem(region.text)

        # Add value polygon to the gtGraphicView
        self.addItem(region.gtValueInside)
        self.addItem(region.gtValueOutside)

    # ---------- Click methods ----------

    def mouseClickEvent(self, ev):
        #super().mouseClickEvent(ev) open context menu with right click, so we need to override it
        # Keep the menu enabled, but with a different button
        if ev.button() == Qt.MouseButton.MiddleButton and self.menuEnabled():
            ev.accept()  # Indicate that the event will be handled so the parent won't receive it
            self.raiseContextMenu(ev)
        else:
            # Convert pixel position into scene position
            p = self.mapSceneToView(ev.scenePos())  # QPointF

            # Search for a region under clicked point
            for region in self.regions:
                if region is not None and (region.IsPointInRegionPolygon(p.x(), p.y()) or
                                           region.text.IsPointInTextPolygon(p.x(), p.y())):
                    ev.accept()
                    self.RegionClickEvent(ev, region)
                    break

    def RegionClickEvent(self, ev, region):
        if ev.button() == Qt.MouseButton.LeftButton:
            self.graphicFilter = True
            print("Region click event 1")
        elif ev.button() == Qt.MouseButton.RightButton:
            self.graphicFilter = True
            print("Region click event 2")


class GTGraphic_Widget(QWidget):
    connGraphicView: ConnGraphicView
    graph_info: ConnGraph_Infos

    def __init__(self, parent=None):
        super().__init__(parent)

        self.graph_info = ConnGraph_Infos()

        self.setContentsMargins(0, 0, 0, 0)

        self.setStyleSheet("border: 1px solid black")

        # red, white, blue
        colorSet = [QColor(255, 0, 0), QColor(200, 200, 200), QColor(0, 0, 255)]

        self.connGraphicView = ConnGraphicView(colorSet)

        self.graphic = pg.GraphicsView()
        self.graphic.setCentralItem(self.connGraphicView)

        QVBoxLayout(self)
        self.layout().addWidget(self.graphic)

    # To force the widget to stay square
    def resizeEvent(self, event):
        size = min(event.size().width(), event.size().height())

        self.resize(QSize(size, size))


# Print iterations progress
# (from https://stackoverflow.com/questions/3173320/text-progress-bar-in-terminal-with-block-characters)
def PrintProgressBar(iteration, total, prefix='', suffix='', decimals=1, length=100, fill='█'):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    sys.stdout.write(f'\r{prefix} |{bar}| {percent}% {suffix}')
    # Print New Line on Complete
    if iteration == total:
        print()


#TODO : remove it later
def printCoordinates(coordinates):
    i = 0
    for point in coordinates:
        print(f"Point {i} : ({point[0]}, {point[1]})")
        i += 1



import os
from pathlib import Path

class ParserPrevious_Files:
    """
    A class to parse Previous File
    """
    def __init__(self):
        
        resourcedir = Path(__file__).parent.parent.parent / 'resources'
        self.backupFile = os.path.join(resourcedir, "Backup_PreviousFiles.txt")
        self.previousFiles = {"DataFiles": {}, "NameFiles": {}, "FlutFiles": [], "FiltersFiles": [], "ProjectFiles": []}

    def LoadFile(self) -> dict:
        """
        Load and parse the backup file.

        Reads the backup file, categorizes lines into different lists or dictionaries based on the category, and handles errors.

        Returns
        -------
        dict
            A dictionary with lists and dictionaries of previous file paths, categorized by type.

        Raises
        ------
        Exception
            If the file cannot be found or the file format is incorrect.
        """
        
        # Ensure file exist (already created or not deleted)
        if os.path.exists(self.backupFile):
            
            try:
                previousFilenames = open(os.path.abspath(self.backupFile), 'r')
            except FileNotFoundError:
                raise Exception("Opening Error - {}".format(self.backupFile))

            category = None
            for line in previousFilenames.readlines():

                # Check the current category
                cleanLines = line.replace("\n", '') 
                if cleanLines == "DataFiles" or cleanLines == "NameFiles" or cleanLines == "FlutFiles" \
                    or cleanLines == "FiltersFiles" or cleanLines == "ProjectFiles":
                    
                    category = cleanLines
                    continue

                # Append File Name in the dictionary
                try:

                    if len(self.previousFiles[category]) == 5:
                        continue

                    # All element before a Category will be ignored
                    if category == "NameFiles" or category == "DataFiles":
                        
                        cleanLinesSepared = cleanLines.split("|")
                        self.previousFiles[category][cleanLinesSepared[0]] = cleanLinesSepared[1]

                    elif category != None:

                        self.previousFiles[category].append(cleanLines)

                except KeyError:
                    raise Exception("Wrong Format Error - Category Error")

            previousFilenames.close()

        return self.previousFiles



import os
import math
import statistics

import networkx as nx

from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QFileDialog, QListWidgetItem, QListWidgetItem, QTableWidgetItem, QAbstractItemView, QHeaderView
from PyQt5.QtCore import Qt, QStandardPaths, QTimer

from src.Controller.LoadingPopup import LoadingPopup

from src.Model.ImportData_Files import ParserData_Files
from src.Model.ImportName_Files import ParserName_Files
from src.Model.ImportFlut_Files import ParserFlut_Files
from src.Model.ImportFilters_Files import ParserFilters_Files

from src.Model.Data_Storage.ConnGraph_Model import ConnGraph_Infos
from src.Model.Data_Storage.ConnGraphSave_Model import ConnGraphSave_Infos
from src.Model.Data_Storage.Filters_Model import Filters_Infos
from src.Model.Data_Storage.FiltersSave_Model import FiltersSave_Infos
from src.Model.Data_Storage.Project_Model import Project_Infos

from copy import deepcopy

class ImportFile_Controller:
    """
        Import File Controller
    """

    connGraph: ConnGraph_Infos
    connGraphSave: ConnGraphSave_Infos
    filters: Filters_Infos
    filtersSave: FiltersSave_Infos
    project: Project_Infos

    def __init__(self, ImportFile_View):

        self.importFile_View = ImportFile_View

        self.connGraph = ConnGraph_Infos()
        self.connGraphSave = ConnGraphSave_Infos()
        self.connGraphSave.LoadCurrentConnGraph(self.connGraph.SaveCurrentConnGraph())

        self.filters = Filters_Infos()
        self.filtersSave = FiltersSave_Infos()
        self.filtersSave.LoadCurrentFilters(self.filters.SaveCurrentFilters())

        self.project = Project_Infos()
        self.projectSave = deepcopy(self.project)

        # Loaded File
        self.flutFile: str = None
        self.dataFile: str = None
        self.nameFile: str = None

        self.threshold: float = None
        self.rank: int = None

        # Validation Part
        self.filtersToIgnore = [False, False, False, False, False, False, False]
        self.validation = False

        # Create all actions used by wingets
        self._ActionCreation()

        # Other Initial Value
        self._OtherInit()

        self.connGraph.InitReset()
        self.filters.InitReset()
        self.project.InitReset()


    # ===========================
    # ===== INITIALIZATION ======
    # ===========================

    def _ActionCreation(self):
        """
        Creates actions for file operations and connects them to their corresponding slots.

        Actions created:
        - Open Data File
        - Open Name File
        - Open Flut File
        - Open Filters File
        - Validation
        - Slider and SpinBox value change
        - Table item click
        """

        # Action Creation - Open Data File
        self.OpenDataFile_Qaction = QtWidgets.QAction('ImportDataFile', self.importFile_View)
        self.OpenDataFile_Qaction.triggered.connect(self.OpenDataFile)

        # Action Creation - Open Name File
        self.OpenNameFile_Qaction = QtWidgets.QAction('ImportNameFile', self.importFile_View)
        self.OpenNameFile_Qaction.triggered.connect(self.OpenNameFile)

        # Action Creation - Open Name File
        self.OpenFlutFile_Qaction = QtWidgets.QAction('ImportFlutFile', self.importFile_View)
        self.OpenFlutFile_Qaction.triggered.connect(self.OpenFlutFile)

        # Action Creation - Open Filters File
        self.OpenFiltersFile_Qaction = QtWidgets.QAction('ImportFiltersFile', self.importFile_View)
        self.OpenFiltersFile_Qaction.triggered.connect(self.OpenFiltersFile)

        # Action Creation - Data/Filter Validation
        self.Validation_Qaction = QtWidgets.QAction('Validation', self.importFile_View)
        self.Validation_Qaction.triggered.connect(self.Validation)

        # Action when Slider / Spin value change
        self.importFile_View.threshold_Slider.sliderReleased.connect(self.SliderValue)
        self.importFile_View.thresholdValue_DoubleSpinBox.valueChanged.connect(self.SpinValue)

        # Action when row in the Table is clicked
        self.importFile_View.percentageTable_TableWidget.itemClicked.connect(self.TablePercentageValue)
        self.importFile_View.nbTable_TableWidget.itemClicked.connect(self.TableNbValue)

    def _OtherInit(self):
        """
        Initializes file lists and sets up item double-click actions.
        """

        # Widget List Initialization

        # For each File (Data / Flut / Filters / Project)
        for dataFile_fullPath, fileType in self.project.previousFiles["DataFiles"].items():

            # Check if the Data File is associated with a Name File
            if dataFile_fullPath in self.project.previousFiles["NameFiles"]:

                nameFile = self.project.previousFiles["NameFiles"][dataFile_fullPath]
                newItem = QListWidgetItem("".join(["\u21A6    (Data File) ", os.path.basename(dataFile_fullPath), "\n", \
                                                    "\u21AA    (Name File) ", os.path.basename(nameFile)]))
                newItem.setData(Qt.UserRole, [dataFile_fullPath, nameFile])
                newItem.setToolTip("".join(["\u21A6 (Data File) ", dataFile_fullPath, "\n", \
                                            "\u21AA (Name File) ", nameFile]))

            else:
                newItem = QListWidgetItem("\u21A6    ({}) {}".format(fileType, os.path.basename(dataFile_fullPath)))
                newItem.setData(Qt.UserRole, [dataFile_fullPath])
                newItem.setToolTip("\u21A6 ({}) {}".format(fileType, dataFile_fullPath))

            self.importFile_View.previousDataFiles_List.addItem(newItem)

        for flutFile in self.project.previousFiles["FlutFiles"]:

            newItem = QListWidgetItem("\u21A6    {}".format(os.path.basename(flutFile)))
            newItem.setData(Qt.UserRole, flutFile)
            newItem.setToolTip(flutFile)
            self.importFile_View.previousFlutFiles_List.addItem(newItem)

        for filtersFile in self.project.previousFiles["FiltersFiles"]:

            newItem = QListWidgetItem("\u21A6    {}".format(os.path.basename(filtersFile)))
            newItem.setData(Qt.UserRole, filtersFile)
            self.importFile_View.previousFiltersFiles_List.addItem(newItem)

        # Other Initialization
        self.importFile_View.limitedGraph_TabWidget.setCurrentIndex(0)

        # Makes All Label in the QListWidget double Clickable
        self.importFile_View.previousDataFiles_List.itemDoubleClicked.connect(self.PreviousDataFile_Click)
        self.importFile_View.previousFlutFiles_List.itemDoubleClicked.connect(self.PreviousFlutFile_Click)
        self.importFile_View.previousFiltersFiles_List.itemDoubleClicked.connect(self.PreviousFiltersFile_Click)


    # ======================
    # ===== Open Files =====
    # ======================

    def OpenDataFile(self):
        """
        Opens a file dialog to load a data file (MAT files, text files).
        
        Calls:
            LoadData(fileName): if a file is selected.
        """

        userDirectory = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
        fileName, _ = QFileDialog.getOpenFileName(self.importFile_View,
                                                "Open Data File",
                                                userDirectory,
                                                "Supported Files (*.mat *.txt)")

        if fileName:
            self.LoadData(fileName)

    def OpenNameFile(self):
        """
        Opens a file dialog to load a name file (TXT files).
        
        Calls:
        - LoadName(fileName) if a file is selected.
        """

        userDirectory = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
        fileName, _ = QFileDialog.getOpenFileName(self.importFile_View,
                                                "Open Name File",
                                                userDirectory,
                                                "Supported Files (*.txt)")

        if fileName:
            self.LoadName(fileName)

    def OpenFlutFile(self):
        """
        Opens a file dialog to load a flut file (FLUT files).
        
        Calls:
            LoadFlut(fileName) if a file is selected.
        """

        userDirectory = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
        fileName, _ = QFileDialog.getOpenFileName(self.importFile_View,
                                                "Open Flut File",
                                                userDirectory,
                                                "Supported Files (*.flut)")

        if fileName:
            self.LoadFlut(fileName)

    def OpenFiltersFile(self):
        """
        Opens a file dialog to load a filters file (JSON files) if both data and flut files are loaded.
        
        Shows an error if data or flut files are missing.
        
        Calls:
            LoadFilters(fileName) if a file is selected and both data and flut files are available.
        """

        if not self.dataFile or not self.flutFile:
            self.importFile_View.errorFiltersFile_Label.show()
            self.importFile_View.errorFiltersFile_Label.setText("Require Data and FLUT")
            return

        userDirectory = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
        fileName, _ = QFileDialog.getOpenFileName(self.importFile_View,
                                                "Open Filters File",
                                                userDirectory,
                                                "Supported Files (*.json)")

        if fileName:
            self.LoadFilters(fileName)


    # =========================
    # ===== Update Object =====
    # =========================

    def UpdateConnGraph(self):
        """
        Updates the connection graph by removing edges with values below the threshold and updating graph information.
        """

        nxGraph = nx.Graph()
        for areaInfoName in self.connGraph.areaInfos.keys():
            nxGraph.add_node(areaInfoName)

        edgesValuesCopy = deepcopy(self.connGraph.edgesValues)
        for id_source, DestValue in edgesValuesCopy.items():
                
            for id_dest, value in DestValue.items():

                if abs(value) < self.threshold:
                    del self.connGraph.edgesValues[id_source][id_dest]
                    continue

            if len(self.connGraph.edgesValues[id_source]) == 0:
                del self.connGraph.edgesValues[id_source]

        self.connGraph.SetGraph(nxGraph)

        edgesValues_withoutDuplicata_Copy = deepcopy(self.connGraph.edgesValues_withoutDuplicata)
        for nodeIDnodeDest, value in edgesValues_withoutDuplicata_Copy.items():
            if abs(value) < self.threshold:
                del self.connGraph.edgesValues_withoutDuplicata[nodeIDnodeDest]

        self.connGraph.numberOfEdges = len(self.connGraph.edgesValues_withoutDuplicata)
        
    def UpdateFilters(self):
        """
        Updates filter settings based on current graph and threshold values.
        """

        # Initialize Filters Object if the attribut isn't None
        self.filters.valueRound = self.valueRound
        self.filters.thresholdPostFiltering = self.threshold

        if not self.filters.weightBetween_threshold[1]:
            self.filters.SetWeightMax(self.connGraph.minMax[1])
        if not self.filters.weightBetween_threshold[0]:
            self.filters.SetWeightMin(self.connGraph.minMax[0])

        self.filters.SetAbsWeightMin(self.RoundInitialization(self.threshold))
        if not self.filters.absWeightBetween_threshold[1]:
            self.filters.SetAbsWeightMax(self.connGraph.absMinMax[1])

        self.filters.SetRankMax(self.rank)
        self.filters.SetRankMin(0)

        # Only one filter of this type can be active at the same time
        if self.filters.discardWeight:
            self.filters.discardRank = self.filters.discardAbsWeight = False
        if self.filters.discardAbsWeight:
            self.filters.discardRank = self.filters.discardWeight = False
        if self.filters.discardRank:
            self.filters.discardWeight = self.filters.discardAbsWeight = False

    def UpdateProject(self):
        """
        Updates the project with the current data, name, and flut files.
        """

        self.project.currentDataFile = self.dataFile
        self.project.currentNameFile = self.nameFile
        self.project.currentFlutFile = self.flutFile


    # ======================
    # ===== Validation =====
    # ======================

    def Validation(self):
        """
        Validates the presence of required files and initiates loading process if validation passes.
        Shows warnings or error messages as needed.
        """

        # Handle Data File Missing
        if self.dataFile is None:
            self.importFile_View.errorValidation_Label.show()
            self.importFile_View.errorValidation_Label.setText("Data File Missing")
            return

        # Handle FLUT File Missing
        elif self.flutFile is None:
            self.importFile_View.errorValidation_Label.show()
            self.importFile_View.errorValidation_Label.setText("FLUT File Missing")
            return

        # Handle Name File Missing
        elif self.nameFile is None:
            self.importFile_View.errorValidation_Label.show()
            self.importFile_View.errorValidation_Label.setText("Name File Missing")
            return

        if len(self.connGraph.areasOrder) > 150:
            self.importFile_View.WarningPopUpNumberRegion()

        self.loadingPopup = LoadingPopup(self.importFile_View)
        self.loadingPopup.show()
        QTimer.singleShot(100, self.LoadingExecute)

    def LoadingExecute(self):
        """
        Performs long-running operations, including updating graph, filters, and project.
        Finalizes the loading process and updates the main window.
        """

        # Perform long-running operations
        self.connGraph.SetTypeConnexion()
        self.UpdateConnGraph()
        self.UpdateFilters()
        self.UpdateProject()

        self.validation = True

        # Finalize operations
        self.importFile_View.parent().mainWindow_controller.LoadDataMainWindow()
        self.loadingPopup.accept()
        self.importFile_View.close()

    def HideRegions(self):
        """
        Updates the area order and information by removing unused areas.
        """

        idName_Inversed = {str(name): ID for ID, name in self.connGraph.idName.items()}
        newAreaOrder = []

        # Delete unused area between Data File and Flut File
        for item in self.connGraph.areasOrder:
            if item[0] in idName_Inversed or str(item[1]) in idName_Inversed:
                newAreaOrder.append(item)
            elif item[0] == 'xxxx':
                newAreaOrder.append(item)

        self.connGraph.areasOrder = newAreaOrder
        self.connGraph.areaInfos = self.limitedAreaInfo

        self.connGraph.numberOfNodes = len(self.connGraph.areaInfos)

    def DisplayAllRegions(self):
        """
        Displays all regions.
        Do nothing (don't delete anything)
        """

        # Display all regions
        pass


    # ==================================
    # ===== Previous Files Actions =====
    # ==================================

    def PreviousDataFile_Click(self, item: QListWidgetItem):
        """
        Load data and optionally name file based on the clicked item.

        Parameters
        ----------
        item : QListWidgetItem
            The item representing the data file and optionally name file.
        """
        
        # Recovert the full path
        dataItem = item.data(Qt.UserRole)

        # Len = 1 -> Only Data File
        if len(dataItem) == 1:
            self.LoadData(dataItem[0])

        # Len = 2 -> Data File + Name File
        else:
            self.LoadData(dataItem[0])
            self.LoadName(dataItem[1])

    def PreviousFlutFile_Click(self, item: QListWidgetItem):
        """
        Load the FLUT file based on the clicked item.

        Parameters
        ----------
        item : QListWidgetItem
            The item representing the FLUT file.
        """
        
        # Recovert the full path
        self.LoadFlut(item.data(Qt.UserRole))

    def PreviousFiltersFile_Click(self, item: QListWidgetItem):
        """
        Load the filters file if data and FLUT files are already loaded.

        Parameters
        ----------
        item : QListWidgetItem
            The item representing the filters file.
        """

        if not self.dataFile or not self.flutFile:
            self.importFile_View.errorFiltersFile_Label.show()
            self.importFile_View.errorFiltersFile_Label.setText("Require Data and FLUT")

        else:
            # Recovert the full path
            self.LoadFilters(item.data(Qt.UserRole))


    # ==========================
    # ===== FILES LOADING ======
    # ==========================

    def LoadData(self, fileName: str):
        """
        Load and parse the data file, and initialize the graph.

        Parameters
        ----------
        fileName : str
            The path to the data file.

        Raises
        ------
        Exception
            If the parser raise an Exception
        """

        self.importFile_View.errorDataNameFile_Label.hide()
        self.importFile_View.errorValidation_Label.setText("")

        try:
            # Parser the File
            parserDataFile = ParserData_Files()
            parserDataFile.GraphCreation(fileName)

        except Exception as exception:
            print("[ERROR]", exception.args[0])
            self.importFile_View.errorDataNameFile_Label.show()
            self.importFile_View.errorDataNameFile_Label.setText(exception.args[0])
            return
        
        if self.flutFile:

            # Verify the coherence between Data and FLUT Files
            if not self.CoherenceVerification_NAMEFLUT():

                # Popup for warning
                self.importFile_View.WarningPopUpDATAFLUT()

        self.dataFile = fileName
        self.nameFile = None

        # Update Display Label
        self.UpdateOpenedDataFile(fileName)
        self.UpdatePreviousDataFiles(fileName)
        self.importFile_View.currentDataFile_Label.setText(os.path.basename(fileName))

        # Graph Initialization and Display Graph Informations
        self.SetUpGraphValues()
        self.GraphPreparation()
        self.InformationsInitialization()

    def LoadName(self, fileName: str):
        """
        Load and parse the name file, and verify coherence with data file.

        Parameters
        ----------
        fileName : str
            The path to the name file.

        Raises
        ------
        Exception
            If the parser raise an Exception
        """

        self.importFile_View.errorDataNameFile_Label.hide()
        self.importFile_View.errorValidation_Label.setText("")

        try:
            self.parserNameFile = ParserName_Files()
            self.parserNameFile.NameFile_Parser(fileName)
            self.CoherenceVerification_NAMEDATA()

        except Exception as exception:
            print("[ERROR]", exception.args[0])
            self.importFile_View.errorDataNameFile_Label.show()
            self.importFile_View.errorDataNameFile_Label.setText(exception.args[0])
            return

        self.nameFile = fileName
        self.project.previousFiles["NameFiles"][self.dataFile] = self.nameFile

        # Update other element
        self.DisplayImportNamePart()
        self.UpdateOpenedNameFile(fileName)
        self.UpdatePreviousNameFiles(fileName)

        # Update other element (Current File Infos)
        previousFileName_basename = os.path.basename(self.dataFile)
        fileName_basename = os.path.basename(fileName)

        self.importFile_View.currentDataFile_Label.setText("".join([previousFileName_basename, "\n", fileName_basename]))

    def CoherenceVerification_NAMEDATA(self):
        """
        Verify the coherence between data and name files.

        Raises
        ------
        ValueError
            If the number of names and nodes do not match.
        """

        # Verify the correct relation between the Connectivity Matrix and Names
        if self.connGraph.numberOfNodesInData != len(self.connGraph.idName):
            raise Exception("Wrong Format - Different Number of Name/Nodes")

    def LoadFlut(self, fileName: str):
        """
        Load and parse the FLUT file, and verify coherence with data file.

        Parameters
        ----------
        fileName : str
            The path to the FLUT file.

        Raises
        ------
        Exception
            If the parser raise an Exception
        """

        self.importFile_View.errorFlutFile_Label.hide()
        self.importFile_View.errorValidation_Label.setText("")

        try:
            
            # Parser the File
            parserFlutFile = ParserFlut_Files()
            parserFlutFile.FlutFile_Parser(fileName)

        except Exception as exception:
            print("[ERROR]", exception.args[0])
            self.importFile_View.errorFlutFile_Label.show()
            self.importFile_View.errorFlutFile_Label.setText(exception.args[0])
            return

        if self.dataFile:
            # Verify the coherence between Data and FLUT Files
            if not self.CoherenceVerification_NAMEFLUT():

                # Popup for warning
                self.importFile_View.WarningPopUpDATAFLUT()

                if not self.flutValidation:
                    return
            
        self.flutFile = fileName

        # Update other element (Last File Opened)
        self.UpdateOpenedFlutFile(fileName)
        self.UpdatePreviousFlutFiles(fileName)

        self.importFile_View.currentFlutFile_Label.setText(os.path.basename(fileName))
        self.importFile_View.nbNodesValue_Label.setText(str(self.connGraph.numberOfNodes))
    
    def CoherenceVerification_NAMEFLUT(self):
        """
        Verify coherence between name and FLUT files.

        Returns
        -------
        bool
            True if the coherence is valid, False otherwise.
        """

        # Verify the Correlation between Name and Flut (Name and Number of Name)
        areaInfo_idName = {str(ID['ID']): name for name, ID in self.connGraph.areaInfos.items()}
        self.limitedAreaInfo = {}
        self.dataToRemove = []

        # For each Name, compare with Flut Name
        for id, nameOrID in self.connGraph.idName.items():

            if nameOrID in self.connGraph.areaInfos:
                self.limitedAreaInfo[nameOrID] = self.connGraph.areaInfos[nameOrID]
                continue

            elif nameOrID in areaInfo_idName:
                newName = areaInfo_idName[nameOrID]
                self.connGraph.idName[id] = newName
                self.limitedAreaInfo[newName] = self.connGraph.areaInfos[newName]
                continue

            self.dataToRemove.append(id)
            print("This ID or Name is different between DATA and FLUT", nameOrID)

        return self.dataToRemove == []

    def IgnoreDATAFLUTIncoherences(self):
        """
        Remove incoherences between data and FLUT files.
        """

        for id_source in self.dataToRemove:

            del self.connGraph.idName[id_source]

            if id_source in self.connGraph.edgesValues:
                for id_dest in self.connGraph.edgesValues[id_source]:
                    del self.connGraph.edgesValues[id_dest][id_source]


                    minMax = (min(id_source, id_dest), max(id_source, id_dest))
                    if minMax in self.connGraph.edgesValues_withoutDuplicata:
                        del self.connGraph.edgesValues_withoutDuplicata[minMax]
                    else:
                        del self.connGraph.edgesValues_withoutDuplicata[(minMax[1], minMax[0])]

                del self.connGraph.edgesValues[id_source]

        self.flutValidation = True

    def CancelFLUT(self):
        """
        Cancel the FLUT file and update the UI.
        """

        # Coherence after DataFile Loading
        if self.dataFile:

            self.flutValidation = False

        # Coherence after FlutFile Loading
        if self.flutFile:

            self.flutFile = None

            self.importFile_View.currentFlutFile_Label.setText("")
            self.importFile_View.nbNodesValue_Label.setText("...")
            self.importFile_View.openedFlutFile_Label.setText("No Flut File Opened")

    def LoadFilters(self, fileName: str):
        """
        Load and parse the filters file, and verify coherence with data and FLUT files.

        Parameters
        ----------
        fileName : str
            The path to the filters file.

        Raises
        ------
        Exception
            If the parser raise an Exception
        """

        self.importFile_View.errorFiltersFile_Label.hide()
        self.importFile_View.errorValidation_Label.setText("")

        try:

            # Parser the File
            parserFiltersFile = ParserFilters_Files()
            parserFiltersFile.FiltersFile_Parser(fileName)

        except Exception as exception:
            print("[ERROR]", exception.args[0])
            self.importFile_View.errorFiltersFile_Label.show()
            self.importFile_View.errorFiltersFile_Label.setText(exception.args[0])
            return


        # Verify the coherence between Data and FLUT Files
        if not self.CoherenceVerification_DATAFILTERS():

            # Popup for warning
            self.importFile_View.WarningPopUpFilters(self.filtersToIgnore)

            if not self.filtersIgnored:
                return
            
        self.threshold = self.filters.thresholdPostFiltering
        self.importFile_View.thresholdValue_DoubleSpinBox.setValue(self.threshold)

        # Update other element (Last File Opened)
        self.UpdateOpenedFiltersFile(fileName)
        self.UpdatePreviousFiltersFiles(fileName)

    def CoherenceVerification_DATAFILTERS(self):
        """
        Verify coherence between data and filters files.

        Returns
        -------
        bool
            True if the filters are coherent with the data, False otherwise.
        """

        # Coherence Verification between data and threshold
        if self.filters.thresholdPostFiltering < self.absMinMax[0]:
            self.filtersToIgnore[0] = True

        if self.filters.weightBetween_threshold[0] < self.connGraph.minMax[0]:
            self.filtersToIgnore[1] = True

        if self.filters.weightBetween_threshold[1] != self.connGraph.minMax[1]:
            self.filtersToIgnore[2] = True

        if self.filters.absWeightBetween_threshold[0] < self.connGraph.absMinMax[0]:
            self.filtersToIgnore[3] = True

        if self.filters.absWeightBetween_threshold[1] != self.connGraph.absMinMax[1]:
            self.filtersToIgnore[4] = True

        if self.filters.rankBetween_threshold[1] > self.connGraph.numberOfEdges:
            self.filtersToIgnore[5] = True

        allInterRegConnCorrect = True
        allRegionsNames = []

        for namesPair in self.filters.discardInterRegConn.keys():
            for name in namesPair:

                if name not in self.connGraph.colorMajorRegions:
                    allInterRegConnCorrect = False
                    break

            allRegionsNames.append(name)

            if not allInterRegConnCorrect:
                break

        allRegionsNames = list(set(allRegionsNames))
        if len(allRegionsNames) != len(self.connGraph.colorMajorRegions):
            self.filtersToIgnore[6] = True

        allCombinaisonPossible = True

        for region1 in allRegionsNames:
            for region2 in allRegionsNames:
                if (region1, region2) not in self.filters.discardInterRegConn:
                    allCombinaisonPossible = False
                    break

            if not allCombinaisonPossible:
                break

        if not allInterRegConnCorrect or not allCombinaisonPossible:
            self.filtersToIgnore[6] = True

        return self.filtersToIgnore == [False, False, False, False, False, False, False]

    def IgnoreFiltersIncoherences(self):
        """
        Ignore incoherences in filters.
        """

        self.filtersIgnored = True

        for indexFilter, toIgnore in enumerate(self.filtersToIgnore):
            if toIgnore:
                match indexFilter:
                    case 0:
                        self.filters.thresholdPostFiltering = self.absMinMax[0]
                    case 1:
                        self.filters.weightBetween_threshold[0] = None
                    case 2:
                        self.filters.weightBetween_threshold[1] = None
                    case 3:
                        self.filters.absWeightBetween_threshold[0] = None
                    case 4:
                        self.filters.absWeightBetween_threshold[1] = None
                    case 5:
                        self.filters.rankBetween_threshold[1] = None
                    case 6:
                        self.filters.discardInterRegConn = {}

    def CancelFilters(self):
        """
        Cancel the filters file and reset to default.
        """

        self.filtersIgnored = False

        self.importFile_View.openedFiltersFile_Label.setText("No Filters File Opened (Optional File)")
        self.filters.LoadSaveFilters(self.filtersSave.RecovertFiltersSave())

        self.importFile_View.thresholdValue_DoubleSpinBox.setValue(self.absMinMax[0])


    # ====================================
    # ===== INTERFACE INITIALIZATION =====
    # ====================================

    def RoundInitialization(self, value: float):
        """
        Round the given value based on the current rounding setting.

        Parameters
        ----------
        value : float
            The value to round.

        Returns
        -------
        int or float
            The rounded value.
        """

        return round(value) if self.valueRound == 0 else round(value, self.valueRound)

    def SetUpGraphValues(self):
        """
        Initialize and set up graph values including min, max, and thresholds.
        """

        self.graphValues = list(self.connGraph.GetAllValues())
        self.connGraph.allGraphValues = self.graphValues
        self.sortedGraphValues = sorted(self.graphValues, key=abs)

        nbValues = len(self.graphValues)
        self.valuesRank = {value: nbValues - rank for rank, value in enumerate(self.sortedGraphValues)}
        self.connGraph.valuesRank = self.valuesRank

        # Min / Max
        self.connGraph.minMax = (min(self.graphValues), max(self.graphValues))

        self.plotMinMax = (min(self.graphValues, key=abs), max(self.graphValues, key=abs))
        self.connGraph.plotMinMax = self.plotMinMax

        self.absMinMax = (abs(self.plotMinMax[0]), abs(self.plotMinMax[1]))
        self.connGraph.absMinMax = self.absMinMax
        
        self.threshold = self.absMinMax[0]
        difference = self.absMinMax[1] - self.absMinMax[0]

        if difference < 2:
            self.valueRound = 5
        elif difference < 5:
            self.valueRound = 4
        elif difference < 20:
            self.valueRound = 3
        elif difference < 50:
            self.valueRound = 2
        elif difference < 100:
            self.valueRound = 1
        else:
            self.valueRound = 0

    def GraphPreparation(self):
        """
        Prepare the graph and initialize UI elements related to the graph.
        """
         
        self.importFile_View.graphSection_Widget.show()

        self.DisplayInitialGraph()

        minValue = self.absMinMax[0]
        maxValue = self.absMinMax[1]

        # Set up the Spin / Slider Parameters
        factor = math.pow(10, self.valueRound)
        minValueInt = math.floor(minValue * factor)
        maxValueInt = math.floor(maxValue * factor)

        # Round Value Spin Box
        if self.valueRound == 0:
            roundMinValue = round(minValue)
        else:
            roundMinValue = round(minValue, self.valueRound) - math.pow(10, -(self.valueRound))

        roundMaxValue = round(maxValue, self.valueRound)

        self.importFile_View.threshold_Slider.setRange(minValueInt, maxValueInt)
        self.importFile_View.threshold_Slider.setTickInterval(math.floor(factor))
        self.importFile_View.threshold_Slider.setValue(minValueInt)

        minText = f"Min :<span style='font-weight: bold; text-decoration: none;'> {roundMinValue}</span>"
        self.importFile_View.minSlider_Label.setText(minText)

        maxText = f"Max :<span style='font-weight: bold; text-decoration: none;'> {roundMaxValue}</span>"
        self.importFile_View.maxSlider_Label.setText(maxText)

        self.importFile_View.thresholdValue_DoubleSpinBox.blockSignals(True)
        self.importFile_View.thresholdValue_DoubleSpinBox.setRange(roundMinValue, roundMaxValue)
        self.importFile_View.thresholdValue_DoubleSpinBox.setSingleStep(math.pow(10, -(self.valueRound)))
        self.importFile_View.thresholdValue_DoubleSpinBox.setDecimals(self.valueRound)
        self.importFile_View.thresholdValue_DoubleSpinBox.setValue(roundMinValue)
        self.importFile_View.thresholdValue_DoubleSpinBox.blockSignals(False)

        self.PercentageTableLoading()
        self.NbTableLoading()

    def PercentageTableLoading(self):
        """
        Load and display percentage-based data in the table widget.
        """

        self.importFile_View.percentageTable_TableWidget.setRowCount(0)  # Reset Table between File
        self.importFile_View.percentageTable_TableWidget.setColumnCount(2)
        self.importFile_View.percentageTable_TableWidget.setHorizontalHeaderLabels(['Percentage', 'Corresponding\nValue'])
        self.importFile_View.percentageTable_TableWidget.setSelectionBehavior(QAbstractItemView.SelectRows)  # Action when Row clicked
        self.importFile_View.percentageTable_TableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)  # Disable Modification
        self.importFile_View.percentageTable_TableWidget.verticalHeader().setVisible(False)

        header = self.importFile_View.percentageTable_TableWidget.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.Stretch)
        header.setSectionResizeMode(1, QHeaderView.Stretch)

        sizeValues = len(self.graphValues)
        sortedValues = sorted(self.graphValues, reverse=True, key=abs)
        absSortedValues = [abs(value) for value in sortedValues]

        # Insert Row in the Table
        percentages = [1, 10, 25, 50, 75, 90, 95]
    
        for indexPercentage, percentage in enumerate(percentages):
            self.importFile_View.percentageTable_TableWidget.insertRow(indexPercentage)
            
            itemPercent = QTableWidgetItem(f"{percentage}%")
            itemPercent.setData(Qt.UserRole, absSortedValues[math.ceil(sizeValues * (percentage / 100)) - 1])  # Graph Threshold
            itemPercent.setTextAlignment(Qt.AlignCenter)
            self.importFile_View.percentageTable_TableWidget.setItem(indexPercentage, 0, itemPercent)

            itemFmValue = absSortedValues[math.ceil(sizeValues * (percentage / 100)) - 1]
            itemFm = QTableWidgetItem(str(self.RoundInitialization(itemFmValue)))
            itemFm.setTextAlignment(Qt.AlignCenter)
            self.importFile_View.percentageTable_TableWidget.setItem(indexPercentage, 1, itemFm)

    def NbTableLoading(self):
        """
        Load and display rank-based data in the table widget.
        """

        # Fill Number Table
        self.importFile_View.nbTable_TableWidget.setRowCount(0)  # Reset Table between File
        self.importFile_View.nbTable_TableWidget.setColumnCount(2)
        self.importFile_View.nbTable_TableWidget.setHorizontalHeaderLabels(['Rank', 'Corresponding\nValue'])
        self.importFile_View.nbTable_TableWidget.setSelectionBehavior(QAbstractItemView.SelectRows)  # Action when Row clicked
        self.importFile_View.nbTable_TableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)  # Disable Modification
        self.importFile_View.nbTable_TableWidget.verticalHeader().setVisible(False)

        header = self.importFile_View.nbTable_TableWidget.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.Stretch)
        header.setSectionResizeMode(1, QHeaderView.Stretch)

        sizeValues = len(self.graphValues)
        sortedValues = sorted(self.graphValues, reverse=True, key=abs)
        absSortedValues = [abs(value) for value in sortedValues]

        ranks = [10, 25, 50, 100, 200, 300, 400, 500]

        # Insert Row in the Table
        for indexRank, threshold in enumerate(ranks):
            if sizeValues > threshold:
                self.importFile_View.nbTable_TableWidget.insertRow(indexRank)
                
                itemNb = QTableWidgetItem(f"{threshold}th")
                itemNb.setData(Qt.UserRole, absSortedValues[threshold - 1])  # Graph Threshold
                itemNb.setTextAlignment(Qt.AlignCenter)
                self.importFile_View.nbTable_TableWidget.setItem(indexRank, 0, itemNb)

                itemFmValue = absSortedValues[threshold - 1]
                itemFm = QTableWidgetItem(str(self.RoundInitialization(itemFmValue)))
                itemFm.setTextAlignment(Qt.AlignCenter)
                self.importFile_View.nbTable_TableWidget.setItem(indexRank, 1, itemFm)

    def InformationsInitialization(self):
        """
        Initialize and display various information metrics related to the graph.
        """

        # Nb Nodes / Connections
        if not self.connGraph.numberOfNodes:
            self.importFile_View.nbNodesValue_Label.setText("...")
        else:
            self.importFile_View.nbNodesValue_Label.setText(str(self.connGraph.numberOfNodes))
        self.importFile_View.nbConnectionsValue_Label.setText(str(self.connGraph.numberOfEdges))

        # ABS Mean / ABS Sum
        if len(self.graphValues) < 2:
            self.importFile_View.meanAbsValue_Label.setText("None")
            self.importFile_View.sumAbsValue_Label.setText("None")
        else:
            meanAbs = statistics.fmean(map(abs, self.graphValues))
            sumAbs = sum(map(abs, self.graphValues))

            self.importFile_View.meanAbsValue_Label.setText(str(self.RoundInitialization(meanAbs)))
            self.importFile_View.sumAbsValue_Label.setText(str(self.RoundInitialization(sumAbs)))

        if self.valueRound == 0:
            roundMinValue = round(self.plotMinMax[0])
        else:
            roundMinValue = round(self.plotMinMax[0], self.valueRound) - math.pow(10, -(self.valueRound))

        self.importFile_View.minValue_Label.setText(str(roundMinValue))
        self.importFile_View.maxValue_Label.setText(str(self.RoundInitialization(self.plotMinMax[1])))

        # Standard Deviation
        if len(self.graphValues) < 2:
            self.importFile_View.standardDeviationValue_Label.setText("None")
        else:
            stddev = statistics.stdev(self.graphValues)
            self.importFile_View.standardDeviationValue_Label.setText(str(self.RoundInitialization(stddev)))

        # Rank
        self.rank = self.connGraph.numberOfEdges
        self.importFile_View.rankValue_Label.setText(str(self.rank))


    # =============================
    # ===== INTERFACE ACTIONS =====
    # =============================

    def UpdateOpenedDataFile(self, fileName: str):
        """
        Update the label with the name of the opened data file.

        Parameters
        ----------
        fileName : str
            The name of the opened data file.
        """

        fileName_basename = os.path.basename(fileName)
        self.importFile_View.openedDataFile_Label.setText("Opened Data File : {}".format(fileName_basename))

    def UpdateOpenedFlutFile(self, fileName: str):
        """
        Update the label with the name of the opened FLUT file.

        Parameters
        ----------
        fileName : str
            The name of the opened FLUT file.
        """

        fileName_basename = os.path.basename(fileName)
        self.importFile_View.openedFlutFile_Label.setText("Opened FLUT File : {}".format(fileName_basename))

    def UpdateOpenedFiltersFile(self, fileName: str):
        """
        Update the label with the name of the opened filters file.

        Parameters
        ----------
        fileName : str
            The name of the opened filters file.
        """

        fileName_basename = os.path.basename(fileName)
        self.importFile_View.openedFiltersFile_Label.setText("Opened Filters File : {}".format(fileName_basename))

    def UpdateOpenedNameFile(self, fileName: str):
        """
        Update the label with the name of the opened name file.

        Parameters
        ----------
        fileName : str
            The name of the opened name file.
        """

        fileName_basename = os.path.basename(fileName)
        self.importFile_View.openedNameFile_Label.setText("Opened Name File : {}".format(fileName_basename))

    def UpdatePreviousDataFiles(self, fileName: str):
        """
        Update the list of previously opened data files.

        Parameters
        ----------
        fileName : str
            The name of the data file to add to the list.
        """

        secondaryFileType = None

        # Hide or Display Name File Part if the Data File doesn't associate name with value
        if not self.connGraph.idName:
            self.DisplayImportNamePart()
            secondaryFileType = "DataFile"
        else:
            self.HideImportNamePart()
            secondaryFileType = "DataNameFile"
            self.nameFile = False  # Don't need Name File

        # Check if the File isn't already in the list
        if fileName not in self.project.previousFiles["DataFiles"]:

            if len(self.project.previousFiles["DataFiles"]) == 5:
                firstItem = self.importFile_View.previousDataFiles_List.item(0)
                del self.project.previousFiles["DataFiles"][firstItem.data(Qt.UserRole)[0]]
                self.importFile_View.previousDataFiles_List.takeItem(self.importFile_View.previousDataFiles_List.row(firstItem))

            fileName_basename = os.path.basename(fileName)
            self.project.previousFiles["DataFiles"][fileName] = secondaryFileType

            # Add the new File
            newItem = QListWidgetItem("\u21A6    ({}) {}".format(secondaryFileType, fileName_basename))
            newItem.setData(Qt.UserRole, [fileName])
            newItem.setToolTip("\u21A6 ({}) {}".format(secondaryFileType, fileName))
            self.importFile_View.previousDataFiles_List.addItem(newItem)

    def UpdatePreviousNameFiles(self, fileName: str):
        """
        Update the list of previously opened name files.

        Parameters
        ----------
        fileName : str
            The name of the name file to add to the list.
        """

        # Iterate through all items in the QListWidget
        for index in range(self.importFile_View.previousDataFiles_List.count()):

            # Get the item at the current index
            itemDataList = self.importFile_View.previousDataFiles_List.item(index)

            # Check if the item data matches the data_to_find
            if itemDataList.data(Qt.UserRole)[0] == self.dataFile:

                previousFileName_basename = os.path.basename(self.dataFile)
                fileName_basename = os.path.basename(fileName)

                itemDataList.setText("".join(["\u21A6    (Data File) ", previousFileName_basename, "\n", \
                                      "\u21AA    (Name File) ", fileName_basename]))
                itemDataList.setData(Qt.UserRole, [self.dataFile, fileName])
                itemDataList.setToolTip("".join(["\u21A6 (Data File) ", self.dataFile, "\n", \
                                                 "\u21AA (Name File) ", fileName]))

    def UpdatePreviousFlutFiles(self, fileName: str):
        """
        Update the list of previously opened FLUT files.

        Parameters
        ----------
        fileName : str
            The name of the FLUT file to add to the list.
        """

        # Check if the File isn't already in the list
        if fileName not in self.project.previousFiles["FlutFiles"]:
            if len(self.project.previousFiles["FlutFiles"]) == 5:
                firstItem = self.importFile_View.previousFlutFiles_List.item(0)
                self.project.previousFiles["FlutFiles"].remove(firstItem.data(Qt.UserRole))
                self.importFile_View.previousFlutFiles_List.takeItem(self.importFile_View.previousFlutFiles_List.row(firstItem))

            # Add the new File
            newItem = QListWidgetItem("\u21A6    {}".format(os.path.basename(fileName)))
            newItem.setData(Qt.UserRole, fileName)
            self.importFile_View.previousFlutFiles_List.addItem(newItem)
            self.project.previousFiles["FlutFiles"].append(fileName)

    def UpdatePreviousFiltersFiles(self, fileName: str):
        """
        Update the list of previously opened filters files.

        Parameters
        ----------
        fileName : str
            The name of the filters file to add to the list.
        """

        # Check if the File isn't already in the list
        if fileName not in self.project.previousFiles["FiltersFiles"]:

            if len(self.project.previousFiles["FiltersFiles"]) == 5:
                firstItem = self.importFile_View.previousFiltersFiles_List.item(0)
                self.project.previousFiles["FiltersFiles"].remove(firstItem.data(Qt.UserRole))
                self.importFile_View.previousFiltersFiles_List.takeItem(self.importFile_View.previousFiltersFiles_List.row(firstItem))

            # Add the new File
            newItem = QListWidgetItem("\u21A6    {}".format(os.path.basename(fileName)))
            newItem.setData(Qt.UserRole, fileName)
            self.importFile_View.previousFiltersFiles_List.addItem(newItem)
            self.project.previousFiles["FiltersFiles"].append(fileName)

    def DisplayImportNamePart(self):
        """
        Display the UI components related to the name file.
        """

        self.importFile_View.openedNameFile_Label.setText("No Name File Opened")
        self.importFile_View.openedNameFile_Label.show()
        self.importFile_View.importNameFile_Button.show()

    def HideImportNamePart(self):
        """
        Hide the UI components related to the name file.
        """

        self.importFile_View.openedNameFile_Label.hide()
        self.importFile_View.importNameFile_Button.hide()

    def SliderValue (self):
        """
        Update the threshold value based on the slider position.
        """

        factor = math.pow(10, self.valueRound)
        sliderValue = self.importFile_View.threshold_Slider.value() / factor

        # Return the minimal value instead of compute the logarithm value
        if sliderValue > self.absMinMax[0] and sliderValue < self.absMinMax[1]:

            # Change the value in logarithm scale
            sliderValue = self.absMinMax[0] * (self.absMinMax[1] / self.absMinMax[0]) \
                        ** (sliderValue / (self.absMinMax[1] - self.absMinMax[0]))
    
        # Re-Adjust
        if sliderValue > self.absMinMax[1]:
            sliderValue = self.absMinMax[1]
        
        # Update Value
        self.importFile_View.thresholdValue_DoubleSpinBox.blockSignals(True)
        self.importFile_View.thresholdValue_DoubleSpinBox.setValue(sliderValue)
        self.importFile_View.thresholdValue_DoubleSpinBox.blockSignals(False)

        # Update Threshold (Filters)
        self.threshold = sliderValue

        # Update Graph
        self.UpdateGraph(sliderValue)
        self.UpdateRank(sliderValue)

    def SpinValue(self):
        """
        Update the slider position based on the spin box value.
        """

        spinValue = self.importFile_View.thresholdValue_DoubleSpinBox.value()
        logValueInversed = (self.absMinMax[1] - self.absMinMax[0]) \
                    * (math.log(spinValue / self.absMinMax[0]) / math.log(self.absMinMax[1] / self.absMinMax[0]))
        
        # Update Value
        self.importFile_View.threshold_Slider.setValue(int(logValueInversed * math.pow(10, self.valueRound)))

        # Update Threshold (Filters)
        self.threshold = spinValue

        # Update Graph
        self.UpdateGraph(spinValue)
        self.UpdateRank(spinValue)

    def UpdateRank(self, newValue: float):
        """
        Update and display the rank based on the new threshold value.

        Parameters
        ----------
        newValue : float
            The new threshold value to determine the rank.
        """

        self.rank = self.connGraph.numberOfEdges
        for value in self.sortedGraphValues:
            if abs(newValue) <= abs(value):
                break

            self.rank -= 1

        # Display the rank associate to the newValue
        self.importFile_View.rankValue_Label.setText(str(self.rank))

    def TablePercentageValue(self, item: QTableWidgetItem):
        """
        Update the threshold value based on the selected percentage table item.

        Parameters
        ----------
        item : QTableWidgetItem
            The item in the percentage table whose value is used to update the threshold.
        """

        self.importFile_View.nbTable_TableWidget.clearSelection()
        row = item.row()

        # Update Threshold (Filters)
        percentageValue = float(self.importFile_View.percentageTable_TableWidget.item(row, 0).data(Qt.UserRole))
        self.threshold = percentageValue

        self.importFile_View.thresholdValue_DoubleSpinBox.setValue(percentageValue)

        # Update Graph
        self.UpdateGraph(percentageValue)

    def TableNbValue(self, item: QTableWidgetItem):
        """
        Update the threshold value based on the selected number table item.

        Parameters
        ----------
        item : QTableWidgetItem
            The item in the number table whose value is used to update the threshold.
        """

        self.importFile_View.percentageTable_TableWidget.clearSelection()
        row = item.row()

        # Update Threshold (Filters)
        nbValue = float(self.importFile_View.nbTable_TableWidget.item(row, 0).data(Qt.UserRole))
        
        self.threshold = nbValue

        self.importFile_View.thresholdValue_DoubleSpinBox.setValue(nbValue)
        
        # Update Graph 
        self.UpdateGraph(nbValue)


    # ==================================
    # ===== Graph Display / Update =====
    # ==================================

    def DisplayInitialGraph(self):
        """
        Display the initial graph with sorted values and annotations for min and max values.
        
        Clears the previous plot, creates a new one, sorts and plots the values, 
        sets the y-axis to a logarithmic scale, adds title and labels, and 
        annotates the min and max values outside the plot area.
        """

        # Clear the previous plot and Create a new one
        self.importFile_View.graph.clear()
        self.graph = self.importFile_View.graph.add_subplot(111)

        # Sort Values -> Initial value
        positiveValues = [value for value in self.graphValues if value > 0]
        negativeValues = [value for value in self.graphValues if value < 0]

        sortedValues = sorted(negativeValues, reverse=True) + sorted(positiveValues, reverse=True)

        x_coords = list(range(len(sortedValues)))
        y_coords = [abs(val) for val in sortedValues]

        self.graph.plot(x_coords, y_coords, label='Initial Curve')

        # Limited plot (initially empty)
        self.limitedCurve, = self.graph.plot(x_coords, y_coords, label='Limited Curve')

        # Set the y-axis to logarithmic scale
        self.graph.set_yscale('log')

        # Add title and labels
        self.graph.set_title('Connectivities Representative Curves\n(in absolute values)', fontsize=15)
        self.graph.set_ylabel('Absolute Value', fontsize=15)

        # Remove x-tick labels
        self.graph.set_xticks([])
        self.graph.set_xticklabels([])

        self.graph.tick_params(axis='y', which='major', labelsize=10)
        self.graph.legend(fontsize=15)

        minValue = self.absMinMax[0]
        maxValue = self.absMinMax[1]

        # Add min and max value annotations outside the plot area
        if type(minValue) == int:
            roundMinValue = minValue
        else:
            roundMinValue = round(minValue, self.valueRound) - math.pow(10, -(self.valueRound))

        roundMaxValue = round(maxValue, self.valueRound)

        # Add min and max value outside the plot area with a white background
        self.graph.text(
            -0.1, -0.05,  # Position near the bottom of the plot
            f'Min: {roundMinValue}', 
            fontsize=9, color='black',
            verticalalignment='bottom', 
            horizontalalignment='left',
            bbox=dict(facecolor='white', edgecolor='black', boxstyle='round,pad=0.3'),
            transform=self.graph.transAxes
        )

        self.graph.text(
            -0.1, 1.05,
            f'Max: {roundMaxValue}', 
            fontsize=9, color='black',
            verticalalignment='top', 
            horizontalalignment='left',
            bbox=dict(facecolor='white', edgecolor='black', boxstyle='round,pad=0.3'),
            transform=self.graph.transAxes
        )

        # Draw the canvas
        self.importFile_View.canvas.draw()

    def UpdateGraph(self, newValue: float):
        """
        Update the graph with the new threshold value, adjusting the displayed curve 
        based on the current threshold.

        Parameters
        ----------
        newValue : float
            The new threshold value used to filter and update the displayed curve.
        """
        
        # Update the limited curve data
        positiveValuesLimited = [value for value in self.graphValues if value > 0]
        negativeValuesLimited = [value for value in self.graphValues if value < 0]

        sortedValuesLimited = sorted(negativeValuesLimited, reverse=True) + sorted(positiveValuesLimited, reverse=True)

        x_coordsLimited = list(range(len(sortedValuesLimited)))
        y_coordsLimited = [abs(value) if abs(value) > newValue - math.pow(10, -(self.valueRound)) 
                            else None for value in sortedValuesLimited]

        # If only one point selected -> display a point 
        if y_coordsLimited.count(None) == len(y_coordsLimited) - 1:
            self.limitedCurve.set_data(x_coordsLimited, y_coordsLimited)
            self.limitedCurve.set_marker('o')
            self.limitedCurve.set_markersize(4)

        else:
            self.limitedCurve.set_data(x_coordsLimited, y_coordsLimited)
            self.limitedCurve.set_marker('')

        # Update the canvas
        self.graph.relim()
        self.graph.autoscale_view()
        self.importFile_View.canvas.draw()

    # =============================
    # ===== INTERFACE CLOSING =====
    # =============================

    def CloseImportWithoutValidation(self):
        """
        Close the import process without validating changes, resetting filters, 
        connection graph, and project to their saved states.
        """

        if not self.validation:

            # ===== Reset Filters =====
            self.filters.LoadSaveFilters(self.filtersSave.RecovertFiltersSave())

            # ===== Reset Conn Graph =====
            self.connGraph.LoadSaveConnGraph(self.connGraphSave.RecovertConnGraphSave())

            # ===== Reset Project
            self.project.InitRestore(self.projectSave)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\ressources\QT_ui\MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_IACOB(object):
    def setupUi(self, IACOB):
        IACOB.setObjectName("IACOB")
        IACOB.resize(1736, 976)
        self.centralWidget = QtWidgets.QWidget(IACOB)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralWidget.sizePolicy().hasHeightForWidth())
        self.centralWidget.setSizePolicy(sizePolicy)
        self.centralWidget.setObjectName("centralWidget")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.centralWidget)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.mainTabWidget = QtWidgets.QTabWidget(self.centralWidget)
        self.mainTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.mainTabWidget.setObjectName("mainTabWidget")
        self.tab1FileInfo_widget = QtWidgets.QWidget()
        self.tab1FileInfo_widget.setObjectName("tab1FileInfo_widget")
        self.gridLayout = QtWidgets.QGridLayout(self.tab1FileInfo_widget)
        self.gridLayout.setObjectName("gridLayout")
        self.fileInfosSupport_Frame = QtWidgets.QFrame(self.tab1FileInfo_widget)
        self.fileInfosSupport_Frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.fileInfosSupport_Frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fileInfosSupport_Frame.setObjectName("fileInfosSupport_Frame")
        self.gridLayout_19 = QtWidgets.QGridLayout(self.fileInfosSupport_Frame)
        self.gridLayout_19.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_19.setObjectName("gridLayout_19")
        self.informationsSection_Widget = QtWidgets.QFrame(self.fileInfosSupport_Frame)
        self.informationsSection_Widget.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.informationsSection_Widget.setFrameShadow(QtWidgets.QFrame.Raised)
        self.informationsSection_Widget.setObjectName("informationsSection_Widget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.informationsSection_Widget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.openedFiles_Widget = QtWidgets.QWidget(self.informationsSection_Widget)
        self.openedFiles_Widget.setObjectName("openedFiles_Widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.openedFiles_Widget)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.openedFiles_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.openedFiles_Label.setObjectName("openedFiles_Label")
        self.horizontalLayout_2.addWidget(self.openedFiles_Label)
        self.openedDataFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.openedDataFile_Label.setObjectName("openedDataFile_Label")
        self.horizontalLayout_2.addWidget(self.openedDataFile_Label)
        self.currentDataFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.currentDataFile_Label.setText("")
        self.currentDataFile_Label.setObjectName("currentDataFile_Label")
        self.horizontalLayout_2.addWidget(self.currentDataFile_Label)
        self.openedFlutFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.openedFlutFile_Label.setObjectName("openedFlutFile_Label")
        self.horizontalLayout_2.addWidget(self.openedFlutFile_Label)
        self.currentFlutFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.currentFlutFile_Label.setText("")
        self.currentFlutFile_Label.setObjectName("currentFlutFile_Label")
        self.horizontalLayout_2.addWidget(self.currentFlutFile_Label)
        self.horizontalLayout_2.setStretch(0, 2)
        self.horizontalLayout_2.setStretch(1, 1)
        self.horizontalLayout_2.setStretch(2, 4)
        self.horizontalLayout_2.setStretch(3, 1)
        self.horizontalLayout_2.setStretch(4, 4)
        self.verticalLayout_3.addWidget(self.openedFiles_Widget)
        self.fileInformations_Widget = QtWidgets.QWidget(self.informationsSection_Widget)
        self.fileInformations_Widget.setObjectName("fileInformations_Widget")
        self.gridLayout_18 = QtWidgets.QGridLayout(self.fileInformations_Widget)
        self.gridLayout_18.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout_18.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_18.setObjectName("gridLayout_18")
        self.nbNodesValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbNodesValue_Label.setText("")
        self.nbNodesValue_Label.setObjectName("nbNodesValue_Label")
        self.gridLayout_18.addWidget(self.nbNodesValue_Label, 0, 2, 1, 1)
        self.nbNodes_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbNodes_Label.setObjectName("nbNodes_Label")
        self.gridLayout_18.addWidget(self.nbNodes_Label, 0, 0, 1, 1)
        self.nbConnections_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbConnections_Label.setObjectName("nbConnections_Label")
        self.gridLayout_18.addWidget(self.nbConnections_Label, 1, 0, 1, 1)
        self.sumAbsValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.sumAbsValue_Label.setText("")
        self.sumAbsValue_Label.setObjectName("sumAbsValue_Label")
        self.gridLayout_18.addWidget(self.sumAbsValue_Label, 0, 4, 1, 1)
        self.standardDeviation_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.standardDeviation_Label.setObjectName("standardDeviation_Label")
        self.gridLayout_18.addWidget(self.standardDeviation_Label, 0, 9, 1, 1)
        self.max_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.max_Label.setObjectName("max_Label")
        self.gridLayout_18.addWidget(self.max_Label, 0, 5, 1, 1)
        self.standardDeviationValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.standardDeviationValue_Label.setText("")
        self.standardDeviationValue_Label.setObjectName("standardDeviationValue_Label")
        self.gridLayout_18.addWidget(self.standardDeviationValue_Label, 0, 10, 1, 1)
        self.sumAbs_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.sumAbs_Label.setObjectName("sumAbs_Label")
        self.gridLayout_18.addWidget(self.sumAbs_Label, 0, 3, 1, 1)
        self.nbConnectionsValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbConnectionsValue_Label.setText("")
        self.nbConnectionsValue_Label.setObjectName("nbConnectionsValue_Label")
        self.gridLayout_18.addWidget(self.nbConnectionsValue_Label, 1, 2, 1, 1)
        self.meanAbsValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.meanAbsValue_Label.setText("")
        self.meanAbsValue_Label.setObjectName("meanAbsValue_Label")
        self.gridLayout_18.addWidget(self.meanAbsValue_Label, 1, 4, 1, 1)
        self.maxValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.maxValue_Label.setText("")
        self.maxValue_Label.setObjectName("maxValue_Label")
        self.gridLayout_18.addWidget(self.maxValue_Label, 0, 6, 1, 1)
        self.meanAbs_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.meanAbs_Label.setObjectName("meanAbs_Label")
        self.gridLayout_18.addWidget(self.meanAbs_Label, 1, 3, 1, 1)
        self.thresholdValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.thresholdValue_Label.setText("")
        self.thresholdValue_Label.setObjectName("thresholdValue_Label")
        self.gridLayout_18.addWidget(self.thresholdValue_Label, 1, 10, 1, 1)
        self.minValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.minValue_Label.setText("")
        self.minValue_Label.setObjectName("minValue_Label")
        self.gridLayout_18.addWidget(self.minValue_Label, 1, 6, 1, 1)
        self.min_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.min_Label.setObjectName("min_Label")
        self.gridLayout_18.addWidget(self.min_Label, 1, 5, 1, 1)
        self.threshold_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.threshold_Label.setObjectName("threshold_Label")
        self.gridLayout_18.addWidget(self.threshold_Label, 1, 9, 1, 1)
        self.maxRelValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.maxRelValue_Label.setText("")
        self.maxRelValue_Label.setObjectName("maxRelValue_Label")
        self.gridLayout_18.addWidget(self.maxRelValue_Label, 0, 8, 1, 1)
        self.minRel_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.minRel_Label.setObjectName("minRel_Label")
        self.gridLayout_18.addWidget(self.minRel_Label, 0, 7, 1, 1)
        self.maxRel_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.maxRel_Label.setObjectName("maxRel_Label")
        self.gridLayout_18.addWidget(self.maxRel_Label, 1, 7, 1, 1)
        self.minRelValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.minRelValue_Label.setText("")
        self.minRelValue_Label.setObjectName("minRelValue_Label")
        self.gridLayout_18.addWidget(self.minRelValue_Label, 1, 8, 1, 1)
        self.verticalLayout_3.addWidget(self.fileInformations_Widget)
        self.verticalLayout_3.setStretch(0, 1)
        self.verticalLayout_3.setStretch(1, 2)
        self.gridLayout_19.addWidget(self.informationsSection_Widget, 0, 0, 1, 1)
        self.graphSection_Widget = QtWidgets.QWidget(self.fileInfosSupport_Frame)
        self.graphSection_Widget.setObjectName("graphSection_Widget")
        self.gridLayout_17 = QtWidgets.QGridLayout(self.graphSection_Widget)
        self.gridLayout_17.setObjectName("gridLayout_17")
        self.graphDisplay_Layout = QtWidgets.QGridLayout()
        self.graphDisplay_Layout.setObjectName("graphDisplay_Layout")
        self.gridLayout_17.addLayout(self.graphDisplay_Layout, 0, 0, 1, 1)
        self.gridLayout_19.addWidget(self.graphSection_Widget, 1, 0, 1, 1)
        self.gridLayout_19.setRowStretch(0, 1)
        self.gridLayout_19.setRowStretch(1, 5)
        self.gridLayout.addWidget(self.fileInfosSupport_Frame, 0, 0, 1, 1)
        self.noInfoTab1_Label = QtWidgets.QLabel(self.tab1FileInfo_widget)
        self.noInfoTab1_Label.setObjectName("noInfoTab1_Label")
        self.gridLayout.addWidget(self.noInfoTab1_Label, 1, 0, 1, 1)
        self.mainTabWidget.addTab(self.tab1FileInfo_widget, "")
        self.tab2Circular_widget = QtWidgets.QWidget()
        self.tab2Circular_widget.setObjectName("tab2Circular_widget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab2Circular_widget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.circularTabSupport_Frame = QtWidgets.QFrame(self.tab2Circular_widget)
        self.circularTabSupport_Frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.circularTabSupport_Frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.circularTabSupport_Frame.setObjectName("circularTabSupport_Frame")
        self.gridLayout_16 = QtWidgets.QGridLayout(self.circularTabSupport_Frame)
        self.gridLayout_16.setObjectName("gridLayout_16")
        self.gridLayout_5.addWidget(self.circularTabSupport_Frame, 0, 0, 1, 1)
        self.noInfoTab2_Label = QtWidgets.QLabel(self.tab2Circular_widget)
        self.noInfoTab2_Label.setObjectName("noInfoTab2_Label")
        self.gridLayout_5.addWidget(self.noInfoTab2_Label, 1, 0, 1, 1)
        self.mainTabWidget.addTab(self.tab2Circular_widget, "")
        self.tab3Pie_widget = QtWidgets.QWidget()
        self.tab3Pie_widget.setObjectName("tab3Pie_widget")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.tab3Pie_widget)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.noInfoTab3_Label = QtWidgets.QLabel(self.tab3Pie_widget)
        self.noInfoTab3_Label.setObjectName("noInfoTab3_Label")
        self.gridLayout_13.addWidget(self.noInfoTab3_Label, 2, 0, 1, 1)
        self.pieTabWidget = QtWidgets.QTabWidget(self.tab3Pie_widget)
        self.pieTabWidget.setObjectName("pieTabWidget")
        self.tabPie1Region_widget = QtWidgets.QWidget()
        self.tabPie1Region_widget.setObjectName("tabPie1Region_widget")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.tabPie1Region_widget)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.addGraphic_button = QtWidgets.QPushButton(self.tabPie1Region_widget)
        self.addGraphic_button.setMinimumSize(QtCore.QSize(30, 30))
        self.addGraphic_button.setMaximumSize(QtCore.QSize(30, 30))
        self.addGraphic_button.setObjectName("addGraphic_button")
        self.gridLayout_14.addWidget(self.addGraphic_button, 0, 1, 1, 1)
        self.pieScrollArea = QtWidgets.QScrollArea(self.tabPie1Region_widget)
        self.pieScrollArea.setWidgetResizable(True)
        self.pieScrollArea.setObjectName("pieScrollArea")
        self.regionScrollContent = QtWidgets.QWidget()
        self.regionScrollContent.setGeometry(QtCore.QRect(0, 0, 1303, 828))
        self.regionScrollContent.setObjectName("regionScrollContent")
        self.pieScrollArea.setWidget(self.regionScrollContent)
        self.gridLayout_14.addWidget(self.pieScrollArea, 0, 0, 2, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_14.addItem(spacerItem, 1, 1, 1, 1)
        self.pieTabWidget.addTab(self.tabPie1Region_widget, "")
        self.tabPie2Region_widget = QtWidgets.QWidget()
        self.tabPie2Region_widget.setObjectName("tabPie2Region_widget")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tabPie2Region_widget)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.addGraphic2_button = QtWidgets.QPushButton(self.tabPie2Region_widget)
        self.addGraphic2_button.setMinimumSize(QtCore.QSize(30, 30))
        self.addGraphic2_button.setMaximumSize(QtCore.QSize(30, 30))
        self.addGraphic2_button.setObjectName("addGraphic2_button")
        self.gridLayout_6.addWidget(self.addGraphic2_button, 0, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem1, 1, 1, 1, 1)
        self.pieScrollArea2 = QtWidgets.QScrollArea(self.tabPie2Region_widget)
        self.pieScrollArea2.setWidgetResizable(True)
        self.pieScrollArea2.setObjectName("pieScrollArea2")
        self.regionScrollContent2 = QtWidgets.QWidget()
        self.regionScrollContent2.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.regionScrollContent2.setObjectName("regionScrollContent2")
        self.pieScrollArea2.setWidget(self.regionScrollContent2)
        self.gridLayout_6.addWidget(self.pieScrollArea2, 0, 0, 2, 1)
        self.pieTabWidget.addTab(self.tabPie2Region_widget, "")
        self.tabPie3Region_widget = QtWidgets.QWidget()
        self.tabPie3Region_widget.setObjectName("tabPie3Region_widget")
        self.gridLayout_20 = QtWidgets.QGridLayout(self.tabPie3Region_widget)
        self.gridLayout_20.setObjectName("gridLayout_20")
        self.addGraphic3_button = QtWidgets.QPushButton(self.tabPie3Region_widget)
        self.addGraphic3_button.setMinimumSize(QtCore.QSize(30, 30))
        self.addGraphic3_button.setMaximumSize(QtCore.QSize(30, 30))
        self.addGraphic3_button.setObjectName("addGraphic3_button")
        self.gridLayout_20.addWidget(self.addGraphic3_button, 0, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_20.addItem(spacerItem2, 1, 1, 1, 1)
        self.pieScrollArea3 = QtWidgets.QScrollArea(self.tabPie3Region_widget)
        self.pieScrollArea3.setWidgetResizable(True)
        self.pieScrollArea3.setObjectName("pieScrollArea3")
        self.regionScrollContent3 = QtWidgets.QWidget()
        self.regionScrollContent3.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.regionScrollContent3.setObjectName("regionScrollContent3")
        self.pieScrollArea3.setWidget(self.regionScrollContent3)
        self.gridLayout_20.addWidget(self.pieScrollArea3, 0, 0, 2, 1)
        self.pieTabWidget.addTab(self.tabPie3Region_widget, "")
        self.tabPie4Region_widget = QtWidgets.QWidget()
        self.tabPie4Region_widget.setObjectName("tabPie4Region_widget")
        self.gridLayout_21 = QtWidgets.QGridLayout(self.tabPie4Region_widget)
        self.gridLayout_21.setObjectName("gridLayout_21")
        self.addGraphic4_button = QtWidgets.QPushButton(self.tabPie4Region_widget)
        self.addGraphic4_button.setMinimumSize(QtCore.QSize(30, 30))
        self.addGraphic4_button.setMaximumSize(QtCore.QSize(30, 30))
        self.addGraphic4_button.setObjectName("addGraphic4_button")
        self.gridLayout_21.addWidget(self.addGraphic4_button, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_21.addItem(spacerItem3, 1, 1, 1, 1)
        self.pieScrollArea4 = QtWidgets.QScrollArea(self.tabPie4Region_widget)
        self.pieScrollArea4.setWidgetResizable(True)
        self.pieScrollArea4.setObjectName("pieScrollArea4")
        self.regionScrollContent4 = QtWidgets.QWidget()
        self.regionScrollContent4.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.regionScrollContent4.setObjectName("regionScrollContent4")
        self.pieScrollArea4.setWidget(self.regionScrollContent4)
        self.gridLayout_21.addWidget(self.pieScrollArea4, 0, 0, 2, 1)
        self.pieTabWidget.addTab(self.tabPie4Region_widget, "")
        self.tabPie5Region_widget = QtWidgets.QWidget()
        self.tabPie5Region_widget.setObjectName("tabPie5Region_widget")
        self.gridLayout_22 = QtWidgets.QGridLayout(self.tabPie5Region_widget)
        self.gridLayout_22.setObjectName("gridLayout_22")
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_22.addItem(spacerItem4, 1, 2, 1, 1)
        self.addGraphic5_button = QtWidgets.QPushButton(self.tabPie5Region_widget)
        self.addGraphic5_button.setMinimumSize(QtCore.QSize(30, 30))
        self.addGraphic5_button.setMaximumSize(QtCore.QSize(30, 30))
        self.addGraphic5_button.setObjectName("addGraphic5_button")
        self.gridLayout_22.addWidget(self.addGraphic5_button, 0, 2, 1, 1)
        self.pieScrollArea5 = QtWidgets.QScrollArea(self.tabPie5Region_widget)
        self.pieScrollArea5.setWidgetResizable(True)
        self.pieScrollArea5.setObjectName("pieScrollArea5")
        self.regionScrollContent5 = QtWidgets.QWidget()
        self.regionScrollContent5.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.regionScrollContent5.setObjectName("regionScrollContent5")
        self.pieScrollArea5.setWidget(self.regionScrollContent5)
        self.gridLayout_22.addWidget(self.pieScrollArea5, 0, 1, 2, 1)
        self.pieTabWidget.addTab(self.tabPie5Region_widget, "")
        self.gridLayout_13.addWidget(self.pieTabWidget, 1, 0, 1, 1)
        self.mainTabWidget.addTab(self.tab3Pie_widget, "")
        self.tab4List_widget = QtWidgets.QWidget()
        self.tab4List_widget.setObjectName("tab4List_widget")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab4List_widget)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.listSupport_Frame = QtWidgets.QFrame(self.tab4List_widget)
        self.listSupport_Frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.listSupport_Frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.listSupport_Frame.setObjectName("listSupport_Frame")
        self.gridLayout_23 = QtWidgets.QGridLayout(self.listSupport_Frame)
        self.gridLayout_23.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_23.setObjectName("gridLayout_23")
        self.infoList_tableWidget = QtWidgets.QTableWidget(self.listSupport_Frame)
        self.infoList_tableWidget.setRowCount(0)
        self.infoList_tableWidget.setColumnCount(9)
        self.infoList_tableWidget.setObjectName("infoList_tableWidget")
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.infoList_tableWidget.setHorizontalHeaderItem(8, item)
        self.infoList_tableWidget.horizontalHeader().setDefaultSectionSize(100)
        self.gridLayout_23.addWidget(self.infoList_tableWidget, 0, 0, 1, 1)
        self.gridLayout_4.addWidget(self.listSupport_Frame, 0, 0, 1, 1)
        self.noInfoTab4_Label = QtWidgets.QLabel(self.tab4List_widget)
        self.noInfoTab4_Label.setObjectName("noInfoTab4_Label")
        self.gridLayout_4.addWidget(self.noInfoTab4_Label, 1, 0, 1, 1)
        self.mainTabWidget.addTab(self.tab4List_widget, "")
        self.tab5GT_widget = QtWidgets.QWidget()
        self.tab5GT_widget.setObjectName("tab5GT_widget")
        self.gridLayout_15 = QtWidgets.QGridLayout(self.tab5GT_widget)
        self.gridLayout_15.setObjectName("gridLayout_15")
        self.noInfoTab5_Label = QtWidgets.QLabel(self.tab5GT_widget)
        self.noInfoTab5_Label.setObjectName("noInfoTab5_Label")
        self.gridLayout_15.addWidget(self.noInfoTab5_Label, 1, 0, 1, 1)
        self.tabGT_TableWidget = QtWidgets.QTabWidget(self.tab5GT_widget)
        self.tabGT_TableWidget.setObjectName("tabGT_TableWidget")
        self.tabGT_widget = QtWidgets.QWidget()
        self.tabGT_widget.setObjectName("tabGT_widget")
        self.gridLayout_25 = QtWidgets.QGridLayout(self.tabGT_widget)
        self.gridLayout_25.setObjectName("gridLayout_25")
        self.GTscrollArea_widget = QtWidgets.QScrollArea(self.tabGT_widget)
        self.GTscrollArea_widget.setWidgetResizable(True)
        self.GTscrollArea_widget.setObjectName("GTscrollArea_widget")
        self.regionScrollContentGT = QtWidgets.QWidget()
        self.regionScrollContentGT.setGeometry(QtCore.QRect(0, 0, 98, 28))
        self.regionScrollContentGT.setObjectName("regionScrollContentGT")
        self.GTscrollArea_widget.setWidget(self.regionScrollContentGT)
        self.gridLayout_25.addWidget(self.GTscrollArea_widget, 0, 0, 1, 1)
        self.tabGT_TableWidget.addTab(self.tabGT_widget, "")
        self.tab2GT_widget = QtWidgets.QWidget()
        self.tab2GT_widget.setObjectName("tab2GT_widget")
        self.gridLayout_24 = QtWidgets.QGridLayout(self.tab2GT_widget)
        self.gridLayout_24.setObjectName("gridLayout_24")
        self.gtList_tableWidget = QtWidgets.QTableWidget(self.tab2GT_widget)
        self.gtList_tableWidget.setObjectName("gtList_tableWidget")
        self.gtList_tableWidget.setColumnCount(2)
        self.gtList_tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.gtList_tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.gtList_tableWidget.setHorizontalHeaderItem(1, item)
        self.gridLayout_24.addWidget(self.gtList_tableWidget, 0, 0, 1, 1)
        self.tabGT_TableWidget.addTab(self.tab2GT_widget, "")
        self.gridLayout_15.addWidget(self.tabGT_TableWidget, 0, 0, 1, 1)
        self.mainTabWidget.addTab(self.tab5GT_widget, "")
        self.gridLayout_9.addWidget(self.mainTabWidget, 0, 0, 1, 1)
        IACOB.setCentralWidget(self.centralWidget)
        self.menuBar = QtWidgets.QMenuBar(IACOB)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 1736, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.menuBar.setFont(font)
        self.menuBar.setObjectName("menuBar")
        IACOB.setMenuBar(self.menuBar)
        self.statuBar = QtWidgets.QStatusBar(IACOB)
        self.statuBar.setObjectName("statuBar")
        IACOB.setStatusBar(self.statuBar)
        self.dockWidget = QtWidgets.QDockWidget(IACOB)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidget.sizePolicy().hasHeightForWidth())
        self.dockWidget.setSizePolicy(sizePolicy)
        self.dockWidget.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dockWidget.setWindowTitle("")
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidgetContents.sizePolicy().hasHeightForWidth())
        self.dockWidgetContents.setSizePolicy(sizePolicy)
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.dockWidgetContents)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.info_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.info_groupBox.sizePolicy().hasHeightForWidth())
        self.info_groupBox.setSizePolicy(sizePolicy)
        self.info_groupBox.setObjectName("info_groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.info_groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.resetFilter_button = QtWidgets.QToolButton(self.info_groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.resetFilter_button.sizePolicy().hasHeightForWidth())
        self.resetFilter_button.setSizePolicy(sizePolicy)
        self.resetFilter_button.setCheckable(False)
        self.resetFilter_button.setObjectName("resetFilter_button")
        self.verticalLayout_2.addWidget(self.resetFilter_button)
        self.resetGraphicFilter_button = QtWidgets.QToolButton(self.info_groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.resetGraphicFilter_button.sizePolicy().hasHeightForWidth())
        self.resetGraphicFilter_button.setSizePolicy(sizePolicy)
        self.resetGraphicFilter_button.setObjectName("resetGraphicFilter_button")
        self.verticalLayout_2.addWidget(self.resetGraphicFilter_button)
        self.verticalLayout_2.setStretch(0, 50)
        self.verticalLayout_2.setStretch(1, 50)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.connNum_label = QtWidgets.QLabel(self.info_groupBox)
        self.connNum_label.setObjectName("connNum_label")
        self.gridLayout_2.addWidget(self.connNum_label, 2, 0, 1, 1)
        self.weightSum_label = QtWidgets.QLabel(self.info_groupBox)
        self.weightSum_label.setObjectName("weightSum_label")
        self.gridLayout_2.addWidget(self.weightSum_label, 0, 0, 1, 1)
        self.weightMean_label = QtWidgets.QLabel(self.info_groupBox)
        self.weightMean_label.setObjectName("weightMean_label")
        self.gridLayout_2.addWidget(self.weightMean_label, 1, 0, 1, 1)
        self.weightSumValue_label = QtWidgets.QLabel(self.info_groupBox)
        self.weightSumValue_label.setObjectName("weightSumValue_label")
        self.gridLayout_2.addWidget(self.weightSumValue_label, 0, 1, 1, 1)
        self.weightMeanValue_label = QtWidgets.QLabel(self.info_groupBox)
        self.weightMeanValue_label.setObjectName("weightMeanValue_label")
        self.gridLayout_2.addWidget(self.weightMeanValue_label, 1, 1, 1, 1)
        self.connNumValue_label = QtWidgets.QLabel(self.info_groupBox)
        self.connNumValue_label.setObjectName("connNumValue_label")
        self.gridLayout_2.addWidget(self.connNumValue_label, 2, 1, 1, 1)
        self.gridLayout_2.setColumnStretch(0, 80)
        self.gridLayout_2.setColumnStretch(1, 20)
        self.horizontalLayout.addLayout(self.gridLayout_2)
        self.horizontalLayout.setStretch(0, 50)
        self.horizontalLayout.setStretch(1, 50)
        self.gridLayout_12.addWidget(self.info_groupBox, 0, 0, 1, 2)
        self.rank_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.rank_groupBox.setObjectName("rank_groupBox")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.rank_groupBox)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.discardRank_spinBox = QtWidgets.QSpinBox(self.rank_groupBox)
        self.discardRank_spinBox.setObjectName("discardRank_spinBox")
        self.gridLayout_11.addWidget(self.discardRank_spinBox, 0, 1, 1, 1)
        self.discardRank_checkBox = QtWidgets.QCheckBox(self.rank_groupBox)
        self.discardRank_checkBox.setObjectName("discardRank_checkBox")
        self.gridLayout_11.addWidget(self.discardRank_checkBox, 0, 0, 1, 1)
        self.gridLayout_11.setColumnStretch(0, 85)
        self.gridLayout_12.addWidget(self.rank_groupBox, 1, 1, 1, 1)
        self.connType_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.connType_groupBox.sizePolicy().hasHeightForWidth())
        self.connType_groupBox.setSizePolicy(sizePolicy)
        self.connType_groupBox.setObjectName("connType_groupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.connType_groupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.contralateral_checkBox = QtWidgets.QCheckBox(self.connType_groupBox)
        self.contralateral_checkBox.setObjectName("contralateral_checkBox")
        self.gridLayout_7.addWidget(self.contralateral_checkBox, 0, 0, 1, 1)
        self.ipsilateral_checkBox = QtWidgets.QCheckBox(self.connType_groupBox)
        self.ipsilateral_checkBox.setObjectName("ipsilateral_checkBox")
        self.gridLayout_7.addWidget(self.ipsilateral_checkBox, 0, 1, 1, 1)
        self.homotopic_checkBox = QtWidgets.QCheckBox(self.connType_groupBox)
        self.homotopic_checkBox.setObjectName("homotopic_checkBox")
        self.gridLayout_7.addWidget(self.homotopic_checkBox, 1, 0, 1, 1)
        self.other_checkBox = QtWidgets.QCheckBox(self.connType_groupBox)
        self.other_checkBox.setObjectName("other_checkBox")
        self.gridLayout_7.addWidget(self.other_checkBox, 1, 1, 1, 1)
        self.gridLayout_12.addWidget(self.connType_groupBox, 3, 0, 1, 2)
        self.connSelection_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.connSelection_groupBox.sizePolicy().hasHeightForWidth())
        self.connSelection_groupBox.setSizePolicy(sizePolicy)
        self.connSelection_groupBox.setObjectName("connSelection_groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.connSelection_groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_12.addWidget(self.connSelection_groupBox, 2, 0, 1, 2)
        self.gtView_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.gtView_groupBox.setEnabled(True)
        self.gtView_groupBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.gtView_groupBox.setObjectName("gtView_groupBox")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.gtView_groupBox)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.binary_checkBox = QtWidgets.QCheckBox(self.gtView_groupBox)
        self.binary_checkBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.binary_checkBox.setObjectName("binary_checkBox")
        self.gridLayout_8.addWidget(self.binary_checkBox, 1, 2, 1, 1)
        self.binaryThreshold_SpinBox = QtWidgets.QDoubleSpinBox(self.gtView_groupBox)
        self.binaryThreshold_SpinBox.setObjectName("binaryThreshold_SpinBox")
        self.gridLayout_8.addWidget(self.binaryThreshold_SpinBox, 1, 3, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_8.addItem(spacerItem5, 1, 1, 1, 1)
        self.addGraphGT_button = QtWidgets.QPushButton(self.gtView_groupBox)
        self.addGraphGT_button.setMinimumSize(QtCore.QSize(30, 30))
        self.addGraphGT_button.setMaximumSize(QtCore.QSize(30, 30))
        self.addGraphGT_button.setText("")
        self.addGraphGT_button.setObjectName("addGraphGT_button")
        self.gridLayout_8.addWidget(self.addGraphGT_button, 1, 0, 1, 1)
        self.gridLayout_8.setColumnStretch(0, 10)
        self.gridLayout_8.setColumnStretch(1, 5)
        self.gridLayout_8.setColumnStretch(2, 5)
        self.gridLayout_8.setColumnStretch(3, 5)
        self.gridLayout_12.addWidget(self.gtView_groupBox, 6, 0, 1, 2)
        self.edgeView_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.edgeView_groupBox.sizePolicy().hasHeightForWidth())
        self.edgeView_groupBox.setSizePolicy(sizePolicy)
        self.edgeView_groupBox.setObjectName("edgeView_groupBox")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.edgeView_groupBox)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.widthCoef_label = QtWidgets.QLabel(self.edgeView_groupBox)
        self.widthCoef_label.setObjectName("widthCoef_label")
        self.horizontalLayout_3.addWidget(self.widthCoef_label)
        self.widthCoefValue_spinBox = QtWidgets.QDoubleSpinBox(self.edgeView_groupBox)
        self.widthCoefValue_spinBox.setObjectName("widthCoefValue_spinBox")
        self.horizontalLayout_3.addWidget(self.widthCoefValue_spinBox)
        self.gridLayout_12.addWidget(self.edgeView_groupBox, 4, 0, 1, 1)
        self.weightGroup_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.weightGroup_groupBox.sizePolicy().hasHeightForWidth())
        self.weightGroup_groupBox.setSizePolicy(sizePolicy)
        self.weightGroup_groupBox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.weightGroup_groupBox.setObjectName("weightGroup_groupBox")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.weightGroup_groupBox)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.discardAbsWeight_spinBox = QtWidgets.QDoubleSpinBox(self.weightGroup_groupBox)
        self.discardAbsWeight_spinBox.setObjectName("discardAbsWeight_spinBox")
        self.gridLayout_10.addWidget(self.discardAbsWeight_spinBox, 2, 1, 1, 1)
        self.discardWeight_checkBox = QtWidgets.QCheckBox(self.weightGroup_groupBox)
        self.discardWeight_checkBox.setObjectName("discardWeight_checkBox")
        self.gridLayout_10.addWidget(self.discardWeight_checkBox, 0, 0, 1, 1)
        self.discardAbsWeight_checkBox = QtWidgets.QCheckBox(self.weightGroup_groupBox)
        self.discardAbsWeight_checkBox.setObjectName("discardAbsWeight_checkBox")
        self.gridLayout_10.addWidget(self.discardAbsWeight_checkBox, 2, 0, 1, 1)
        self.discardWeightUp_spinBox = QtWidgets.QDoubleSpinBox(self.weightGroup_groupBox)
        self.discardWeightUp_spinBox.setObjectName("discardWeightUp_spinBox")
        self.gridLayout_10.addWidget(self.discardWeightUp_spinBox, 0, 1, 1, 1)
        self.discardWeightDown_spinBox = QtWidgets.QDoubleSpinBox(self.weightGroup_groupBox)
        self.discardWeightDown_spinBox.setObjectName("discardWeightDown_spinBox")
        self.gridLayout_10.addWidget(self.discardWeightDown_spinBox, 1, 1, 1, 1)
        self.gridLayout_10.setColumnStretch(0, 85)
        self.gridLayout_12.addWidget(self.weightGroup_groupBox, 1, 0, 1, 1)
        self.edgeColor_groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.edgeColor_groupBox.sizePolicy().hasHeightForWidth())
        self.edgeColor_groupBox.setSizePolicy(sizePolicy)
        self.edgeColor_groupBox.setObjectName("edgeColor_groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.edgeColor_groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.colorPalette_comboBox = QtWidgets.QComboBox(self.edgeColor_groupBox)
        self.colorPalette_comboBox.setEditable(False)
        self.colorPalette_comboBox.setObjectName("colorPalette_comboBox")
        self.verticalLayout.addWidget(self.colorPalette_comboBox)
        self.displayColorPalette_Label = QtWidgets.QLabel(self.edgeColor_groupBox)
        self.displayColorPalette_Label.setText("")
        self.displayColorPalette_Label.setObjectName("displayColorPalette_Label")
        self.verticalLayout.addWidget(self.displayColorPalette_Label)
        self.gridLayout_12.addWidget(self.edgeColor_groupBox, 4, 1, 1, 1)
        self.export_Layout = QtWidgets.QGridLayout()
        self.export_Layout.setObjectName("export_Layout")
        self.exportList_button = QtWidgets.QToolButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exportList_button.sizePolicy().hasHeightForWidth())
        self.exportList_button.setSizePolicy(sizePolicy)
        self.exportList_button.setMinimumSize(QtCore.QSize(0, 30))
        self.exportList_button.setMaximumSize(QtCore.QSize(16777215, 30))
        self.exportList_button.setObjectName("exportList_button")
        self.export_Layout.addWidget(self.exportList_button, 0, 0, 1, 1)
        self.exportCircular_button = QtWidgets.QToolButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exportCircular_button.sizePolicy().hasHeightForWidth())
        self.exportCircular_button.setSizePolicy(sizePolicy)
        self.exportCircular_button.setMinimumSize(QtCore.QSize(0, 30))
        self.exportCircular_button.setMaximumSize(QtCore.QSize(16777215, 30))
        self.exportCircular_button.setObjectName("exportCircular_button")
        self.export_Layout.addWidget(self.exportCircular_button, 1, 0, 1, 1)
        self.gridLayout_12.addLayout(self.export_Layout, 5, 0, 1, 2)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_12.addItem(spacerItem6, 7, 0, 1, 2)
        self.gridLayout_12.setRowStretch(0, 10)
        self.gridLayout_12.setRowStretch(1, 10)
        self.gridLayout_12.setRowStretch(2, 100)
        self.gridLayout_12.setRowStretch(3, 10)
        self.gridLayout_12.setRowStretch(4, 10)
        self.gridLayout_12.setRowStretch(5, 10)
        self.gridLayout_12.setRowStretch(6, 10)
        self.gridLayout_12.setRowStretch(7, 100)
        self.dockWidget.setWidget(self.dockWidgetContents)
        IACOB.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dockWidget)

        self.retranslateUi(IACOB)
        self.mainTabWidget.setCurrentIndex(0)
        self.pieTabWidget.setCurrentIndex(0)
        self.tabGT_TableWidget.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(IACOB)

    def retranslateUi(self, IACOB):
        _translate = QtCore.QCoreApplication.translate
        IACOB.setWindowTitle(_translate("IACOB", "MainWindow"))
        self.mainTabWidget.setProperty("class", _translate("IACOB", "whiteGrayTabWidget"))
        self.openedFiles_Label.setText(_translate("IACOB", "File Opened\n"
"(Data / FLUT)"))
        self.openedFiles_Label.setProperty("class", _translate("IACOB", "standardLabel"))
        self.openedDataFile_Label.setText(_translate("IACOB", "Data File :"))
        self.openedDataFile_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.currentDataFile_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.openedFlutFile_Label.setText(_translate("IACOB", "FLUT File :"))
        self.openedFlutFile_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.currentFlutFile_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.nbNodesValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.nbNodes_Label.setText(_translate("IACOB", "Number of Nodes :"))
        self.nbNodes_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.nbConnections_Label.setText(_translate("IACOB", "Number of Connections :"))
        self.nbConnections_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.sumAbsValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.standardDeviation_Label.setText(_translate("IACOB", "Standard\n"
"Deviation"))
        self.standardDeviation_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.max_Label.setText(_translate("IACOB", "Max ABS :"))
        self.max_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.standardDeviationValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.sumAbs_Label.setText(_translate("IACOB", "Sum (ABS) :"))
        self.sumAbs_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.nbConnectionsValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.meanAbsValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.maxValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.meanAbs_Label.setText(_translate("IACOB", "Mean (ABS) :"))
        self.meanAbs_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.thresholdValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.minValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.min_Label.setText(_translate("IACOB", "Min ABS :"))
        self.min_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.threshold_Label.setText(_translate("IACOB", "Threshold :"))
        self.threshold_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.maxRelValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.minRel_Label.setText(_translate("IACOB", "Max REL :"))
        self.minRel_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.maxRel_Label.setText(_translate("IACOB", "Min REL :"))
        self.maxRel_Label.setProperty("class", _translate("IACOB", "infoLabel"))
        self.minRelValue_Label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.noInfoTab1_Label.setText(_translate("IACOB", "No File Loaded"))
        self.noInfoTab1_Label.setProperty("class", _translate("IACOB", "noFileLabel"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab1FileInfo_widget), _translate("IACOB", "File Info"))
        self.noInfoTab2_Label.setText(_translate("IACOB", "No File Loaded"))
        self.noInfoTab2_Label.setProperty("class", _translate("IACOB", "noFileLabel"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab2Circular_widget), _translate("IACOB", "Circular"))
        self.noInfoTab3_Label.setText(_translate("IACOB", "No File Loaded"))
        self.noInfoTab3_Label.setProperty("class", _translate("IACOB", "noFileLabel"))
        self.pieTabWidget.setProperty("class", _translate("IACOB", "whiteGrayTabWidget"))
        self.addGraphic_button.setProperty("class", _translate("IACOB", "whiteBGButton"))
        self.pieTabWidget.setTabText(self.pieTabWidget.indexOf(self.tabPie1Region_widget), _translate("IACOB", "Areas"))
        self.addGraphic2_button.setProperty("class", _translate("IACOB", "whiteBGButton"))
        self.pieTabWidget.setTabText(self.pieTabWidget.indexOf(self.tabPie2Region_widget), _translate("IACOB", "Major Regions"))
        self.addGraphic3_button.setProperty("class", _translate("IACOB", "whiteBGButton"))
        self.pieTabWidget.setTabText(self.pieTabWidget.indexOf(self.tabPie3Region_widget), _translate("IACOB", "Major Regions (bar)"))
        self.addGraphic4_button.setProperty("class", _translate("IACOB", "whiteBGButton"))
        self.pieTabWidget.setTabText(self.pieTabWidget.indexOf(self.tabPie4Region_widget), _translate("IACOB", "Connection Type"))
        self.addGraphic5_button.setProperty("class", _translate("IACOB", "whiteBGButton"))
        self.pieTabWidget.setTabText(self.pieTabWidget.indexOf(self.tabPie5Region_widget), _translate("IACOB", "Connection Type (bar)"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab3Pie_widget), _translate("IACOB", "Pie"))
        self.infoList_tableWidget.setProperty("class", _translate("IACOB", "infoTable"))
        item = self.infoList_tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("IACOB", "Node 1"))
        item = self.infoList_tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("IACOB", "Node 2"))
        item = self.infoList_tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("IACOB", "Label 1"))
        item = self.infoList_tableWidget.horizontalHeaderItem(3)
        item.setText(_translate("IACOB", "Label 2"))
        item = self.infoList_tableWidget.horizontalHeaderItem(4)
        item.setText(_translate("IACOB", "Region 1"))
        item = self.infoList_tableWidget.horizontalHeaderItem(5)
        item.setText(_translate("IACOB", "Region 2"))
        item = self.infoList_tableWidget.horizontalHeaderItem(6)
        item.setText(_translate("IACOB", "Connection type"))
        item = self.infoList_tableWidget.horizontalHeaderItem(7)
        item.setText(_translate("IACOB", "Connectivity"))
        item = self.infoList_tableWidget.horizontalHeaderItem(8)
        item.setText(_translate("IACOB", "Rank"))
        self.noInfoTab4_Label.setText(_translate("IACOB", "No File Loaded"))
        self.noInfoTab4_Label.setProperty("class", _translate("IACOB", "noFileLabel"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab4List_widget), _translate("IACOB", "List"))
        self.noInfoTab5_Label.setText(_translate("IACOB", "No File Loaded"))
        self.noInfoTab5_Label.setProperty("class", _translate("IACOB", "noFileLabel"))
        self.tabGT_TableWidget.setProperty("class", _translate("IACOB", "whiteGrayTabWidget"))
        self.tabGT_TableWidget.setTabText(self.tabGT_TableWidget.indexOf(self.tabGT_widget), _translate("IACOB", "Local"))
        self.gtList_tableWidget.setProperty("class", _translate("IACOB", "infoTable"))
        item = self.gtList_tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("IACOB", "Global Measures"))
        item = self.gtList_tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("IACOB", "Values (ABS Graph)"))
        self.tabGT_TableWidget.setTabText(self.tabGT_TableWidget.indexOf(self.tab2GT_widget), _translate("IACOB", "Global"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.tab5GT_widget), _translate("IACOB", "GT"))
        self.info_groupBox.setTitle(_translate("IACOB", "Info"))
        self.info_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.resetFilter_button.setText(_translate("IACOB", "Reset filter"))
        self.resetFilter_button.setProperty("class", _translate("IACOB", "grayRoundedButton"))
        self.resetGraphicFilter_button.setText(_translate("IACOB", "Reset graphic filter"))
        self.resetGraphicFilter_button.setProperty("class", _translate("IACOB", "grayRoundedButton"))
        self.connNum_label.setText(_translate("IACOB", "Connectivity number :"))
        self.connNum_label.setProperty("class", _translate("IACOB", "filterParamLabel"))
        self.weightSum_label.setText(_translate("IACOB", "Weights sum :"))
        self.weightSum_label.setProperty("class", _translate("IACOB", "filterParamLabel"))
        self.weightMean_label.setText(_translate("IACOB", "Weights mean :"))
        self.weightMean_label.setProperty("class", _translate("IACOB", "filterParamLabel"))
        self.weightSumValue_label.setText(_translate("IACOB", "..."))
        self.weightSumValue_label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.weightMeanValue_label.setText(_translate("IACOB", "..."))
        self.weightMeanValue_label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.connNumValue_label.setText(_translate("IACOB", "..."))
        self.connNumValue_label.setProperty("class", _translate("IACOB", "valueLabel"))
        self.rank_groupBox.setTitle(_translate("IACOB", "Rank Filter"))
        self.rank_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.discardRank_spinBox.setProperty("class", _translate("IACOB", "blackSpinBox"))
        self.discardRank_checkBox.setText(_translate("IACOB", "Discard rank >"))
        self.discardRank_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.connType_groupBox.setTitle(_translate("IACOB", "Type of connection"))
        self.connType_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.contralateral_checkBox.setText(_translate("IACOB", "Contralateral"))
        self.contralateral_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.ipsilateral_checkBox.setText(_translate("IACOB", "Ipsilateral"))
        self.ipsilateral_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.homotopic_checkBox.setText(_translate("IACOB", "Homotopic"))
        self.homotopic_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.other_checkBox.setText(_translate("IACOB", "Other"))
        self.other_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.connSelection_groupBox.setTitle(_translate("IACOB", "Inter-regional connections"))
        self.connSelection_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.gtView_groupBox.setTitle(_translate("IACOB", "GT view"))
        self.gtView_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.binary_checkBox.setText(_translate("IACOB", "Binary"))
        self.binary_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.binaryThreshold_SpinBox.setProperty("class", _translate("IACOB", "blackSpinBox"))
        self.addGraphGT_button.setProperty("class", _translate("IACOB", "grayBGButton"))
        self.edgeView_groupBox.setTitle(_translate("IACOB", "Edge view"))
        self.edgeView_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.widthCoef_label.setText(_translate("IACOB", "Width coefficient :"))
        self.widthCoef_label.setProperty("class", _translate("IACOB", "filterParamLabel"))
        self.widthCoefValue_spinBox.setProperty("class", _translate("IACOB", "blackSpinBox"))
        self.weightGroup_groupBox.setTitle(_translate("IACOB", "Weight Filter"))
        self.weightGroup_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.discardAbsWeight_spinBox.setProperty("class", _translate("IACOB", "blackSpinBox"))
        self.discardWeight_checkBox.setText(_translate("IACOB", "Discard weight >= / <="))
        self.discardWeight_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.discardAbsWeight_checkBox.setText(_translate("IACOB", "Discard abs weight >="))
        self.discardAbsWeight_checkBox.setProperty("class", _translate("IACOB", "blackCheckBox"))
        self.discardWeightUp_spinBox.setProperty("class", _translate("IACOB", "blackSpinBox"))
        self.discardWeightDown_spinBox.setProperty("class", _translate("IACOB", "blackSpinBox"))
        self.edgeColor_groupBox.setTitle(_translate("IACOB", "Edge color"))
        self.edgeColor_groupBox.setProperty("class", _translate("IACOB", "filterGroupBox"))
        self.colorPalette_comboBox.setProperty("class", _translate("IACOB", "blackComboBox"))
        self.displayColorPalette_Label.setProperty("class", _translate("IACOB", "displayEdgesColor"))
        self.exportList_button.setText(_translate("IACOB", "Export List"))
        self.exportList_button.setProperty("class", _translate("IACOB", "grayRoundedButton"))
        self.exportCircular_button.setText(_translate("IACOB", "Export Circular Graph"))
        self.exportCircular_button.setProperty("class", _translate("IACOB", "grayRoundedButton"))
